! ---------------------------------------------------------------------
!
!				PCD2BUFR
!
!			  PROTOTIPO VERSAO 0.8.4
!
!	       GRUPO DE PRE-PROCESSAMENTO DE DADOS DO CPTEC
!
! ---------------------------------------------------------------------
!
!   OBJETIVO :
!
!	Programa para ler os dados das PCDs fornecidos pelo DSA/INPE 
!	em formato texto, e grava-lo em formato BUFR (WMO)
!
!
!  HISTORICO : 	
!
!	VERSAO 0.1 - SERGIO H.S.FERREIRA 11/09/2003 - Apenas para ler
!		   os arquivos do DSA, todos os tipos de PCDs em For-
!		   tran 90.
!
!	VERSAO 0.2 - Waldenio Gambi, de 26/09/2003 a 16/10/2003
!		   Para ler e gravar (em
!		   formato BUFR) O primeiro prototipo. Nao grava
!		   todos os dados disponiveis.
!		   No momento, tambem nao grava os nomes das PCDs,
!		   devido a problemas com as tabelas BUFR do CPTEC
!
!	VERSAO 0.3 - Waldenio Gambi, de 22/10/2003 a 30/10/2003
!		   Incluida a gravacao da secao 2. Ela eh opcional,
!		   mas exigida pelo Metview e BDM
!
!       VERSAO 0.8 - SERGIO H.S.FERREIRA e Luiz Amrico 12/12/2003
!                  Gravao de sada para Grads em mdulo separado
!                  Clculo da precipitao (mm/3h)
!                  Clculo da presso reduzida ao nvel do mar
     
! ---------------------------------------------------------------------
!
!			ESTRUTURA DO PROGRAMA:
!
! ---------------------------------------------------------------------

! ' COMENTARIOS E DEFINICOES
! ' APRESENTACAO DO PROGRAMA
! ' INICIALIZACAO DE VARIAVEIS LOCAIS
! ' ABRE OS ARQUIVOS DE ENTRADA


! ' Abre arquivo de entrada com os dados das PCDs 

! ' Abre arquivo de saida (formato BUFR)
!
! ' Abre e le o arquivo com o cadastro das PCDs
!
! ' Identifica a estacao PCD e pega seus dados: 
! 
!	Le a primeira linha do arquivo de entrada
!	(identificacao e lista de dados)
!
!	Extrai o Nome/Numero da PCD e lista de dados
!	( carrega variavel PCD_ID e VARIAVEL(NDADOS) )
!
!	' Agora procura pela estacao no cadastro de 
!	' identificacao, necessario para os outros valores
!
!	SE (achar a estacao no cadastro)
!
!		ENTAO: 	carrega variavel PCD_ID
!		SENAO: 	grava log "Nao achei estacao NNN no
!			cadastro"
!			FIM - Vai ler Proxima PCD
!	FIM SE
!
! ' Laco para ler os dados e Criar o Registro BUFR.
!
!	LOOP enquanto NOT EOF(ArqPCD)
!
!		Le linha de ArqPCD
!		Extrai os dados para VALORES(NDADOS)
!		Prepara vetor para BUFR (descritores e dados
!			' Carrega R8ARR
!			' Carrega Descrit
!		' CALL GravaBUFR
!
!	FIM LOOP
!
!	Fecha ArqPCD
!
!	Fecha ArqBUFR
!

!
! ---------------------------------------------------------------------
!

program PCD2BUFR



  USE STRINGFLIB
  USE MREADPCD
  USE MGRADSPCD

  implicit none			! Nada de definicoes implicitas !


  ! ---------------------------------------------------------------------
  !
  !	 	       DECLARACOES DE ESTRUTURAS:
  !
  ! ---------------------------------------------------------------------

  type IDENTITY
     Integer:: PCDCOD		! CODIGO DA PCD
     character(len=20)::NAME 	! Nome da Localidade ou Estacao
     Real:: LAT,lON			! Coordenadas da PCD
     real:: ALTITUDE			! Altitude da estacao
  end type IDENTITY

  type Bufr_Table
     integer:: f,x,y,scale,ref,nbits
     character(len=46) :: BUFR_desc
     character(len=13) :: Unit
  end type Bufr_Table
  !
  ! ---------------------------------------------------------------------
  !
  !		 	  VARIAVEIS LOCAIS:
  !
  ! ---------------------------------------------------------------------


  ! type(IDENTITY):: PCD_ID	! Identificacao de uma PCD (Lat,Lon,etc)

  ! integer:: NDADOS		! Numero de informacoes que a PCD em 
  ! questao fornece (depende do tipo de
  ! estacao: meteorologica, hidrologica..)

  ! character(len=20):: VARIAVEL(NDADOS)	! ARRAY que recebe a descricao
  ! dos dados da PCD, que vem na 
  ! 1a linha de cada arquivo

  integer(2) :: status,i,j,argc	! Utilizadas na leitura dos arquivos
  integer :: iargc

  character(len=255):: infile     ! nome de um arquivo de entrada
  character(len=255):: lista      ! nome da lista de arquivos de entrada
  character(len=255):: outfile	  ! nome do arquivo de saida
  character(len=10):: datefile            ! LAC
  character(len=255):: option             ! LAC
  character(len=255), DIMENSION(3):: FLAG ! LAC
  integer(1), DIMENSION(3):: GRAVA        ! LAC

  !* 1. declaracao de variaveis principais do programa
  !
  Integer :: PCDNUM		! Numero da PCD
  integer :: totalpcds          ! Numero Total de PCDS Lidas
  Integer :: TotalPCDsID        ! Numero Total de PCDS identificadas
  Integer :: TotalPCDsDesc      ! Numero Total de PCDs Desconhecidas 
  Integer :: TotalPCDsNoData    ! Numero Total de PCDs Sem Dados no horario
  Integer :: TotalPCDsYesData   ! Numero Total de PCDs Com Dados no horario
  !
  type(MREADPCD_Id):: IdPCD     ! Identificacao de uma PCD (Lat,Lon,etc..)
  type(MREADPCD_Date):: HObs    ! Data e Hora de uma observacao de PCD 
  type(MREADPCD_Obs),dimension(29)::Obs ! Dados Observacionais de uma Observacao
  !
  type(Bufr_Table),dimension(2000)::Table_B ! Tabela Bufr B
  integer :: ct                             ! Tamanho tabela Bufr B
  !
  Integer(1):: TIPOPCD	! Tipo da Pcd (Meteorologica, Hidrologica ou Boia)
  integer  :: Nobs	! Numero de Dados em uma observacoes
  !
  INTEGER(kind=4),DIMENSION(8)::DATE	  !LAC
  INTEGER(kind=4)::DATE_OBS               !LAC
  CHARACTER (12),DIMENSION(3)::REAL_CLOCK !LAC
  !
  !* 2. declaracao de  variaveis auxiliares do programa 
  !
  character(len=255),dimension(10)::c
  integer::nc
  CHARACTER*8 :: STID
  Integer:: NFLAG,NLEV
  REAL::MISSING,VENTO,RLAT,RLON,TIM

  ! ---------------------------------------------------------------------
  !
  !			    INICIO DO PROGRAMA:
  !
  ! ---------------------------------------------------------------------

  !
  ! 	APRESENTACAO DO PROGRAMA:
  !

  PRINT *,' '
  print *,'--------------------------------------------------------'
  PRINT *,'			  PCD2BUFR'
  PRINT *,' '
  PRINT *,'		        Dezembro 2003'
  PRINT *,' '
  print *,'    PROGRAMA PARA PROCESSAMENTO DAS PCDS - CPTEC/INPE'
  PRINT *,' '
  print *,'           GRUPO DE PRE_PROCESSAMENTO DE DADOS '
  print *,'--------------------------------------------------------'
  PRINT *,' '
  PRINT *,' '


  !
  !	INICIALIZACAO DAS VARIAVEIS LOCAIS:
  !

  argc=0
  infile=''
  lista=''
  outfile=''
  totalpcds=0
  TotalPCDsDesc=0
  TotalPCDsNoData=0
  TotalPCDsYesData=0
  
  ! ---------------------------------------------------------------------
  !
  !		        ABRE OS ARQUIVOS DE ENTRADA:
  !
  ! ---------------------------------------------------------------------

  ! Os arquivos de entrada e saida podem ser passados por parametro, por
  ! linha de comando. Se os dois parametros nao forem passados, eh apre-
  ! sentado um "help" de utilizacao.
  !
  ! O Parametro de entrada deve ser um arquivo texto que contenha uma lis-
  ! ta com o nome dos arquivos de entrada. Dessa forma, este programa le /
  ! uma quantidade de arquivos com os dados de varias PCDs, e grava apenas
  ! um arquivo de saida.

  ! Se existirem dois argumentos de entrada, assume que eles sao a lista 
  ! que contem os arquivos de entrada, e o arquivo de saida em BUFR
  ! Caso contrario, apresenta o "help"

  argc =  iargc()		! Pega os argumentos de entrada

  if (argc>2) then 
     !call GetArg(1,lista,status) !So para PC - Windows
     call GetArg(argc-1,lista)        !Maquinas DEC

     !call GetArg(2,outfile,status) !So para PC - Windows
     call GetArg(argc,outfile)         !Maquina DEC
  endif
  !
  CALL DATE_AND_TIME(REAL_CLOCK(1), REAL_CLOCK(1), REAL_CLOCK(3), DATE)
  DATE_OBS = (INT((DATE(5)-DATE(4)/60)/3))*3
  !
  DO i=1, argc-2
     call GetArg(i,option)
     IF (option == '-date') THEN
        j=i
        j=j+1
        call GetArg(j,option)
        DATE(1)=VAL(option)
        j=j+1
        call GetArg(j,option)
        DATE(2)=VAL(option)
        j=j+1
        call GetArg(j,option)
        DATE(3)=VAL(option)
        j=j+1
        call GetArg(j,option)
        DATE_OBS=VAL(option)
        EXIT
     ENDIF
  ENDDO
  !MES,DIA,ANO,HORA,MIN
  Hobs%ANO=DATE(1)
  Hobs%MES=DATE(2)
  Hobs%DIA=DATE(3)
  Hobs%HORA=DATE_OBS
  Hobs%MIN=0
  !
  grava(1) = 0
  grava(2) = 0
  grava(3) = 0
  FLAG(1) = '-bufr'
  FLAG(2) = '-grads'
  FLAG(3) = '-txt'
  !
  DO i=1,3
     DO j=1, argc-2
        call GetArg(j,option)
        if (option==FLAG(i)) grava(i)=1
     ENDDO
  ENDDO
  !    
  print *,'INPUT           ', trim(lista)
  print *,'OUTPUT          ', trim(outfile)
  print *,'BUFR  Y(1) N(0) ', grava(1)
  print *,'GRADS Y(1) N(0) ', grava(2)
  print *,'TXT   Y(1) N(0) ', grava(3)
  print *,'YEAR            ', DATE(1) 
  print *,'MONTH           ', DATE(2)
  print *,'DAY             ', DATE(3)
  print *,'HOUR            ', DATE_OBS
  !
  if ((argc<=2).OR.((grava(1)==0).AND.(grava(2)==0).AND.(grava(3)==0))) then
     print *,'------------------------------------------------'               
     print *,' '
     print *,' Usage:'
     print *,' '
     print *,' > pcd2bufr.x -options input_list output_name '
     print *,' '
     print *,' list_input: file with name of DSA-PCD files'
     print *,' output_name: without extension   '
     print *,' '
     print *,' options: -grads -bufr -txt -date'
     print *,' '
     print *,' date format: yyyy mm dd hh'
     print *,' '
     print *,'------------------------------------------------'
     stop
  end if
  !
  ! -------------------------------------------------------------
  !
  open(9,file=lista,STATUS='old') ! Abre o arquivo das PCDs

  ! ---------------------------------------------------------------------
  !
  !		        ABRE OS ARQUIVOS DE SAIDA:
  !
  ! ---------------------------------------------------------------------
  
  write(datefile, '(i4,i2.2,i2.2,i2.2)') DATE(1),DATE(2),DATE(3),DATE_OBS
  outfile=trim(outfile)//trim(datefile)
  open(2,file=trim(outfile)//'.log',status='unknown') 
  ! Abre o log file

  if (grava(3)==1) open(3,file=trim(outfile)//'.dat',status='unknown') 
  ! Abre o arq. TXT

  if (grava(2)==1) open (15,NAME=trim(outfile)//'.bin',FORM='UNFORMATTED', RECORDTYPE='STREAM') ! Abre arq. Grads

  !write(3,*)'PCD   I       LAT        LON     HA  YYYYMMDDHHNN BUFRC      VALOR'
  ! ---------------------------------------------------------------------
  !
  !   Inicializacoes 
  !
  ! ---------------------------------------------------------------------

  ! ---------------------------------------------------------------------
  !  Tabela B Bufr (WMO)
  ! ---------------------------------------------------------------------

  ct=1
!
  open(23,file='Bufr_Table_B.TXT',status='old')

10 read(23,50,end=888)Table_B(ct)%f,Table_B(ct)%x,Table_B(ct)%y,Table_B(ct)%BUFR_desc,Table_B(ct)%Unit,Table_B(ct)%scale,Table_B(ct)%ref,Table_B(ct)%nbits  
  ct=ct+1
50 format(i1,i2,i3,1x,a46,1x,a13,i4,1x,i11,1x,i3)

  !write(*,101)Table_B(ct)%f,Table_B(ct)%x,Table_B(ct)%y,Table_B(ct)%BUFR_desc,Table_B(ct)%Unit,Table_B(ct)%scale,Table_B(ct)%ref,Table_B(ct)%nbits 
!101 format(i1,'|',i2,'|',i3,'|',a46,'|',a13,'|',i4,'|',i11,'|',i3)
  
  !fxy=Table_B(ct)%f*100000+Table_B(ct)%x*1000+Table_B(ct)%y

  goto 10
888 continue
!
!
!!$ print *, 'Para testar fxy '
!!$  DO i=1,ct
!!$     fxy=Table_B(i)%f*100000+Table_B(i)%x*1000+Table_B(i)%y 
!!$     print *, i, ' ', fxy, ' ', Table_B(i)%f, ' ', Table_B(i)%x, ' ',Table_B(i)%y
!!$  END DO
!
!
  print *, 'Leitura da tabela B (Bufr/WMO): ', ct, ' linhas'
  !
  ! ---------------------------------------------------------------------
  !  Tabela B Bufr (WMO)
  ! ---------------------------------------------------------------------
  !
  ! -------------------------
  call Init_MREADPCD			! inicializacao do sergio.
  ! -------------------------

  ! Le o nome do arquivo com os dados da PCD

20 read(9,'(a)',end=999)infile


  !* 5.1 - Identifica o Numero e a Localizacao da PCD 

  call Sep_NatNum(infile,c,nc) 

  PCDNUM = VAL(C(NC))
  Totalpcds = TotalPcds + 1		

  call IdStation_MREADPCD(PCDNUM,IdPCD)


  !*5.2 - Procede a Leitura dos dados de uma estacao
  !       Caso esta estacao tenha sido identificada na 
  !       tabela de estacoes (idPCD%PCDCOD/=null)

  If (IdPCD%PCDCOD /= Null) then 

     TotalPCDsId = TotalPCDsId + 1

     close(10)
     open(10,file=infile,STATUS='old')	! Abre o arquivo das PCDs...

     Call IdType_MREADPCD(10,TIPOPCD) 	! Identifica o Tipo da PCD 
     ! (Met.,Hidro.ou Boia)


     If (TIPOPCD>0) then          

        Call ReadOBS_MREADPCD(10,TIPOPCD,IdPCD,HObs,Obs,NObs) 
        !
        If (Nobs>0) then
           TotalPCDsYesData=TotalPCDsYesData+1
           !
           !  Registro grads
           !
           if (grava(2)==1) call GRAVA_GRADS(15,IdPCD,Obs,HObs,NObs)
           !
           !  Registro BUFR.
           !
           if (grava(1)==1) Call GRAVABUFR(IdPCD,Obs,HObs,NObs,outfile)
           !
           !if (grava(1)==1) Call GRAVABUFR2(IdPCD,Obs,HObs,NObs,Table_B,ct)
           !
           !  Registro ASCII
           !
           if (grava(3)==1) THEN
              do i=1,nobs
                 write(3,100) Obs(i)%BUFRCOD,Obs(i)%valor,Obs(i)%QC 
100              FORMAT(I8,1X,F12.4,I3)
              end do
           end if
        Else
           write(2,*)'PCD SEM DADOS  =',PCDNUM
           TotalPCDsNoData = TotalPCDsNoData + 1
        endif
     end if

     close(10)

  else 
     write(2,*)'PCD Nao Identificada =',PCDNUM
     TotalPCDsDesc = TotalPCDsDesc + 1

  end if


  goto 20    !* Proximo Arquivo 

999 continue  

  Print *, "Numero Total de PCDs Lidas...........", TotalPcds
  Print *, "Numero Total de PCDs Identificadas...", TotalPCDsId
  Print *, "Numero Total de PCDs Desconhecidas...", TotalPCDsDesc
  Print *, "Numero Total de PCDs Com dados.......", TotalPCDsYesData
  Print *, "Numero Total de PCDs Sem dados.......", TotalPCDsNoData
  !
  close(2)

  ! Encerrar gravacao do arquivo grads

  rlat=0.0
  rlon=0.0
  TIM=0.0
  NLEV=0
  NFLAG=1
  if (grava(2)==1) WRITE (15) STID,RLat,RLON,TIM,NLEV,NFLAG  

  if (grava(2)==1) close(15)

 ! * Gravar descritor do arquivo grads
 
 if (grava(2)==1) call GRAVA_GRADSCTL(trim(outfile),DATE_OBS,DATE(3),DATE(2),DATE(1)) 
     
end program PCD2BUFR


! ---------------------------------------------------------------------
!
!			   SUBROUTINE  GRAVABUFR
!
!			  VERSAO 0.2 - 22/10/2003
!
!	       GRUPO DE PRE-PROCESSAMENTO DE DADOS DO CPTEC
!
! ---------------------------------------------------------------------
!
!  OBJETIVO :
!
!	Esta rotina recebe os valores e dados de uma PCD, e os grava 
!	em formato BUFR (WMO)
!
!  COMENTARIOS: 
!
!	Um arquivo BUFR e um binario, e todos os bytes tem sua posicao
!	exata, nao havendo espaco para linefeeds ou outros  caracteres
!	de controle. Assim, saber como gravar o formato e essebncial !
!
!	Em primeiro lugar, o arquivo de saida deve ser aberto com a op-
!	cao CARRIAGECONTROL='NONE', de forma que nenhum separador de /
!	registro seja gravado entre os diversos WRITE. Outro detalhe :
!	as strings a serem gravadas devem ter o numero exato de  bytes
!	necessarios. Se elas forem maiores, espacos (' ') indesejaveis
!	irao estragar o binario BUFR !!!
!
!  HISTORICO : 	
!
!	VERSAO 0.1 - Waldenio Gambi - O primeiro prototipo. Limitado,
!		   porem funcional.
!			Nesta versao limitada, por problemas na inter-
!			face com o sistema do cptec, nao coloco os
!			nomes das pcds e uso a tabela B000460201
!			(06/10/03)
!
!	VERSAO 0.2 - Waldenio Gambi - Inclui a gravacao da secao 2.
!			Esta secao eh opcional, mas exigida pelo Metview
!			e tambem pelo BDM.
!
! ---------------------------------------------------------------------

!
! PARAMETROS DE ENTRADA IDPCD OBS...
!

 SUBROUTINE GRAVABUFR(IdPCD,Obs,HObs,NObs,outfile)

 USE MREADPCD
     
!
!		 DEFINICOES DAS VARIAVEIS DE ENTRADA
!

 type(MREADPCD_Id), intent(in):: IdPCD			! Identificacao de uma 
 							! PCD (Lat,Lon,etc..)
 type(MREADPCD_Date), intent(in):: HObs      		! Data e Hora de uma 
 							! observacao de PCD 
 type(MREADPCD_Obs),dimension(29), intent(in)::Obs 	! Dados Observacionais 
 							! de uma Observacao
 character(len=*),intent(in):: outfile                  ! LAC
!
!		  DEFINICOES DE VARIAVEIS LOCAIS
!

 character (8)  sec0	! Para gravar a secao 0 (8 bytes)
 character (18) sec1	! Para gravar os 18 bytes da secao 1

 character (52) sec2 	! Nesta versao a secao 2 eh uma copia do
			! mdb_pcd.f, e cheia de valores arbitrados.

 character (7) sec3	! A identificacao da secao 3 tem 7 bytes
 character (2) descrit	! Os descritores sempre tem 2 bytes cada !
 
 character (46) sec4	! No nosso caso, a secao 4 tem 28 bytes. ! *CUIDADO*
 			! Melhor aqui eh gravar byte a byte -> mudar!
 			
 character (4)  sec5	! Para gravar o final do BUFR = '7777'

 INTEGER Tam_BUFR	! Tamanho total do arquivo BUFR (grav. na secao 1)
 INTEGER Tam_sec2	! Numero de bytes da secao 2
 INTEGER Tam_sec3	! Numero de bytes da secao 3
 INTEGER Tam_sec4	! Numero de bytes da secao 4
 
 
 INTEGER Ndescrit	! Numero de descritores utilizados no momento.
 
 INTEGER(kind=1) oct1	! octetos (os bytes) que serao gravados
 INTEGER(kind=1) oct2	! no stream binario. Varios usos.
 INTEGER(kind=1) oct3

 INTEGER(kind=4) MISSING ! missing value em BUFR, todos bits=1
 REAL (kind=4) medida	! Para receber os valores lidos das PCDs
 INTEGER(kind=4) valor	! para diversos usos
 INTEGER(kind=4) rdbmes	! LAC
 INTEGER(kind=4) rdbdia	! LAC
 INTEGER(kind=4) rdbano	! LAC
 INTEGER(kind=4),DIMENSION(8)::rdbtime	 ! LAC
 CHARACTER (12),DIMENSION(3)::RDB_CLOCK !LAC
 CHARACTER (20) AUX	! String auxiliar para diversos usos
 INTEGER i		! Para os loops.
 



!
!		 CALCULA O TAMANHO DO ARQUIVO BUFR
!

 MISSING = HUGE(MISSING)

 Ndescrit = 15+2+2				! No momento, uso 15 descritores
 						! apenas
 						
 Tam_sec2 = 52					! Numero de bytes da secao 2
 Tam_sec3 = 7 + (2*Ndescrit)			! Tamanho secao 3 - descritores
 Tam_sec4 = 4+40+1+1				! Tamanho da secao 4 para estes
 						! descritores. 

 Tam_BUFR = 8 + 18 + 4 				! tamanho Secao 0, 1 e 5
 Tam_BUFR = Tam_BUFR + Tam_sec2
 Tam_BUFR = Tam_BUFR + Tam_sec3			! Tamanho total do registro
 Tam_BUFR = Tam_BUFR + Tam_sec4			! Tamanho secao 4 - os dados.

!
! Descritores BUFR - tabela B - Ver Manual WMO - a lista abaixo contem os
! valores da tabela B para os dados utilizados nesta codificacao. 
!
! Os codigos F e X (2 e 6 bits) sao armazenados no primeiro byte do descri-
! tor, enquanto que o codigo Y no segundo byte.
!
! para codificar, o valor deve ser multiplicado por 10^(escala) e subtraido
! da referencia : Valor = Valor*(10^escala) - referencia
!
! Codigo F/X/Y	Descricao		Unidade	escala	referencia	tamanho (sec4)
!
! 0 01 015 	Station or site Name 	ASCII	 -	 - 		160 bits
! ** este nao esta sendo usado no momento ! - problema tabela bufr.
! 
! 0 04 001 	Ano			ANO	0	0		12 bits
! 0 04 002 	Mes			mes	0	0		 4 bits
! 0 04 003 	Dia			dia	0	0		 6 bits
! 0 04 004 	Hora			hora	0	0		 5 bits
! 0 04 005	Minutos			minuto	0	0		 6 bits
! 0 05 001 	Latitude(high accuracy)	degree	5	-9000000	25 bits
! 0 06 001	Longitude( " )		degree	5	-18000000	26 bits
! 0 07 001	station's height 	meter	0	-400		15 bits
! 0 10 004	Pressao			Pascal	-1	0		14 bits
! 0 11 011	wind direction (at 10M)	degree	0	0		 9 bits
! 0 11 012	wind Speed (at 10M) 	m s-1	1	0		12 bits
! 0 12 001	temperatura dry-bulb 	K	1	0		12 bits
! 0 12 016	temp. max. (24h a 2m) 	K	1	0		12 bits
! 0 12 017	temp. min. (24h a 2m) 	K	1	0		12 bits
! 0 13 003	Relative Humidity	%	0	0		 7 bits
! 0 13 020      Tot prec past 3 hours   kg m-2  1      -1               14 bits
! 0 10 051      Press. red. to msl      Pa      -1      0               14 bits
! 1 01 017      Replicator ----------------------------------------------------
! 0 33 007      Confidence Level        %       0	0		 7 bits
!
!		ABRE ARQUIVO DE SAIDA (PARA APPEND)
!

 open(4,file=trim(outfile)//'.rdb',ORGANIZATION='SEQUENTIAL',POSITION='APPEND',CARRIAGECONTROL='NONE')
 
!
!		        PREPARA E GRAVA A SECAO 0
!

! A variavel Tam_BUFR eh integer*4, sendo que o primeiro bit sera sempre 0,
! pois ela devera sempre ser um numero positivo. Como o tamanho do arquivo
! BUFR eh registrado em 3 bytes, o tamanho maximo do BUFR sera de 23 bits,
! igual a 8388607 bytes.
!
! A questao aqui eh extrair o valor numerico destes bytes para guarda-los
! nas posicoes de 5-7, na secao 0. Eh preciso eliminar o byte mais signifi-
! cativo da variavel Tam_BUFR, que alias devera ser sempre = 0 ! - Qual o
! valor entao, dos bytes 2, 3 e 4 ?
!
! Eh preciso cuidado aqui, por causa do famoso problema do BigEndianXlittle
! Se o valor dos bytes for acessado diretamente da memoria, teremos este
! problema. Formas alternativas de tratar isto sao : usar mascaras de bits,
! operacoes matematicas ou deslocamento de bits. Uma grande vantagem do F90
! eh a conveniente funcao IBITS !!! :-D

 oct3 = IBITS(Tam_BUFR,0,8)			! o menor byte do tamanho
 oct2 = IBITS(Tam_BUFR,8,8)			! o byte do meio
 oct1 = IBITS(Tam_BUFR,16,8)			! o maior byte do tamanho
   
 sec0 = ''					! inicializa string de saida
 sec0 = 'BUFR'					! o indicador
 sec0(5:7) = char(oct1)//char(oct2)//char(oct3)	! Tamanho total do arquivo
 sec0(8:8) = char(2)				! BUFR edicao 2 
  
 write (4, '(a)') sec0 				! Grava secao 0
  
!
!		    PREPARA E GRAVA A SECAO 1
!

 sec1 = ''				! inicializa

 sec1 = char(0)//char(0)//char(18)	! Bytes 1 a 3 contem o tamanho da 
 					! secao 1. Aqui eh sempre = 18 !

 sec1(4:4) = char(0)			! Byte 4 contem a versao da BUFR 
 					! master table. Para meteorologia=0

 sec1(5:6) = char(0)//char(46) 		! Os bytes 5 e 6 contem o codigo do 
					! centro que gerou o arquivo  BUFR,
					! segundo a code table 0 01 031 (ver 
					! manual WMO). No caso, o codigo do 
					! INPE (Brazilian Space Agency !?) 
					! eh 46 (octeto 6)
					
 sec1(7:7) = char(0)			! Update sequence number = 0

 sec1(8:8) = char(128) 			! Bit 1=0 (nao ha secao 2)
 					! os demais bits sao reservados.
 			
 sec1(9:9) = char(0)			! Data category type (BUFR Table A)
 					! 0 corresponde a surface data/land
 					
 sec1(10:10) = char(7)			! Data category sub-type
 					! Por enquanto, 7 para as PCDs, se-
 					! gundo sugestao da  Rosangela  (O 
 					! BDM usa este valor para classifi-
 					! car o dado ?)
 
 sec1(11:11) = char(2)			! Versao utilizada da tabela Master/WMO
 
 sec1(12:12) = char(1)			! Versao utilizada da tabela local/INPE
 
! Um detalhe importante nos proximos bytes (13-18), e o fato de que centro dife-
! rentes estao usando padroes diferentes aqui !!! - A ordem a seguir eh a reco -
! mendada pela WMO, mas o NCEP, por exemplo, poe o ano nos bytes 13 e 14, fazen-
! do um shift nos demais dados e o minuto no byte 18. 
! Devido a estas diferencas, uma boa politica pode ser: nao usar a data que es -
! tiver nesta secao, ou decodificar de acordo com o centro que gerou a mensagem.

 valor = Hobs%ANO - 2000
 sec1(13:13) = char(Valor)		! Ano do Seculo

 Valor = Hobs%MES
 sec1(14:14) = char(Valor)		! Mes da Observacao
 
 Valor = Hobs%Dia
 sec1(15:15) = char(Valor)		! Dia da Observacao

 sec1(16:16) = char(Hobs%HORA)		! Hora da Observacao

 sec1(17:17) = char(Hobs%MIN)		! Minuto da Observacao

 sec1(18:18) = char(0)			! Reservado para uso local - Nao uso !

 write (4, '(a)') sec1			! Grava a Secao 1 


!
!		    PREPARA E GRAVA A SECAO 2
!

 sec2 = ''				! inicializa

 sec2 = char(0)//char(0)//char(Tam_sec2) ! Bytes 1 a 3 contem o tamanho

 sec2(4:4) = char(0)			! Byte 4 eh reservado (=0)

 sec2(5:5) = char(0)			! Ntype com 8 bits. No nosso caso, 0
 					! para estacao superficie.
 
 sec2(6:6) = char(7)			! NSBtype com 8 bits. sugestao da ro:
 					! pcd = 7 
 
! ---------------------------------------------------------------------------
! Agora sim a parte complicada, concatenar a sequencia de bits formando um
! "stream" continuo para ser gravado. 
! --> Puxa ! uma boa parte disto e "copy & paste" da secao 4 !
! ---------------------------------------------------------------------------

 oct1 = IBITS(Hobs%ANO,4,8)	! Este eh o primeiro byte, com os 8 primeiros
 				! bits de informacao de ANO
 				
 				
 oct2 = IBITS(Hobs%ANO,0,4)	! Este recebe os 4 bits restantes de ANO, mas
 				! agora falta concatenar o proximo dado neste
 				! byte, antes de gravar. Entao:
 				
 oct2 = ISHFT(oct2,4)		! desloca esses 4 bits para a esquerda, abrin-
 				! do espaco para colocar os bits do proximo /
 				! dado (mes=4bits)
 				
 oct3 = IBITS(Hobs%MES,0,4)	! Os 4 bits significativos de MES vao p/ oct3

 oct2 = oct2 + oct3		! faz o merge dos bits de ano e mes. Agora 
 				! temos dois bytes completos, em oct1 e oct2,
 				! com 12 bits de ano e 4 bits de mes

 sec2(7:8) = char(oct1)//char(oct2)	! Guarda estes bytes na string sec2.
 

 oct1 = IBITS(Hobs%Dia,0,6)	! Dia com 6 bits
 oct1 = ISHFT(oct1,2)		! desloca 2 bits para a esquerda
 oct2 = IBITS(Hobs%HORA,3,2)	! pega os 2 bits mais significativos de Hora
 				! Hora tem 5 bits, 3 vao para o proximo byte
 oct3 = oct1 + oct2		! completa o byte !!
 
 sec2(9:9) = char(oct3)		! Guarda este byte na string sec2.

 oct1 = IBITS(Hobs%HORA,0,3)	! agora os 3 bits restantes de Hora
 oct1 = ISHFT(oct1,5)		! desloca-os para a esquerda

 oct2 = IBITS(Hobs%MIN,1,5)	! Minutos tem 6 bits. 5 ficam neste byte, e
 				! 1 sobra para o proximo. oct2 pega os 5 
 oct3 = oct1 + oct2		! completa o byte com Hora e Minuto
 
 sec2(10:10) = char(oct3)	! Guarda este byte na string sec2.

 oct1 = IBITS(Hobs%MIN,0,1)	! Pega o menor bit de minuto
 oct1 = ISHFT(oct1,7)		! Com este deslocamento, o ultimo bit de
 				! minutos ja fica na posicao para merge.

 ! A Proxima informacao sao os Segundos (!), com 6 bits, mas para o nosso
 ! caso, pode ser sempre zero. (nao tenho segundos das PCDs.)
 
 ! A seguir, vem mais um bit com 0 (!) talvez apenas para completar o byte.
 ! Assim, podemos gravar o nosso byte.
 
 sec2(11:11) = char(oct1)	! Guarda este byte na string sec2.

! Agora a vez das Longitudes e Latitudes. Nota: aqui eh primeiro a longitude!

! ---------------------------------------------------------------------------
! Longitude, com ajuste de escala e referencia, agora em 26 bits. 
! ---------------------------------------------------------------------------

 Valor = (IdPCD%Lon*10**5)-(-18000000)

 oct1 = IBITS(Valor,18,8)	! Pega os 8 bits mais significativos da
 				! longitude (18-25)

 oct2 = IBITS(valor,10,8)	! Proximos 8 bits da Longitude (10-17)
 
 oct3 = IBITS(valor,2,8)	! Proximos 8 bits da Longitude (2-9)
  
 sec2(12:14) = char(oct1)//char(oct2)//char(oct3)	! Guarda na sec2

! Sobraram 2 bits, e o Proximo valor a ser gravado eh: zero, com 6 bits !!
! portanto, basta gravar os 2 bits, apos deslocar 6 posicoes.

 oct1 = IBITS(valor,0,2)	! Ultimos 2 bits da Longitude (0-1)
 oct1 = ISHFT(oct1,6)		! Deslocamento para Incluir os 6 bits nulos.
 
 sec2(15:15) = char(oct1)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------
! Latitude, com ajuste de escala e
! referencia. A escala eh 5 e referencia -9000000, em 25 bits
! ---------------------------------------------------------------------------
 
 Valor = (IdPCD%Lat*10**5)-(-9000000)

 oct1 = IBITS(Valor,17,8)	! Pega os 8 bits mais significativos da
 				! latitude (17-24)
 sec2(16:16) = char(oct1)	! Guarda este byte na string sec2

 oct1 = IBITS(valor,9,8)	! Proximos 8 bits de Latitude (9-16)
 sec2(17:17) = char(oct1)	! Guarda este byte na string sec2

 oct1 = IBITS(valor,1,8)	! Proximos 8 bits de Latitude (1-8)
 sec2(18:18) = char(oct1)	! Guarda este byte na string sec2

 oct1 = IBITS(valor,0,1)	! Pega o ultimo bit da latitude
 oct1 = ISHFT(oct1,7)		! Com este deslocamento, o ultimo
 				! bit da latitude fica pronto

! Como se adicionam 7 bits 0 (zero), posso gravar direto. 				

 sec2(19:19) = char(oct1)	! Guarda este byte na string sec2

! --------------------------------------------------------------------------
! Agora Gravo o Nome, em 5 bytes (do byte 20 ao 24)
! --------------------------------------------------------------------------

 AUX = '                  '
 
 write(AUX,*) IdPCD%PCDCOD	! Preparo o Nome da PCD, no caso, serah ape-
 				! nas o codigo, que vem no nome do arquivo.
 AUX=adjustl(AUX)		! Aqui eu facco uma implementacao da funcao 
 				! STRING$ do BASIC ! i.e. converto o numero 
 				! em string

 sec2(20:36) = AUX(1:5) 	! Station Number, com 5 bytes

! -------------------------------
! 2 Bytes com outra informacao... (qual ?)
! -------------------------------

 
 sec2(37:38) = char(0)//char(Tam_BUFR)	! ILenght - 16 bits Tamanho Bufr ?!

! --------------------------------------------------------------------------
  !  Agora a hora em que o boletim foi escrito no RDB (faz sentido no CPTEC?)
  ! --------------------------------------------------------------------------

  ! mais 3 bytes, temporariamente com zeros....

  CALL DATE_AND_TIME(RDB_CLOCK(1), RDB_CLOCK(1), RDB_CLOCK(3), rdbtime)  
  rdbano=rdbtime(1)
  rdbmes=rdbtime(2) 
  rdbdia=rdbtime(3)

  oct1 = IBITS(rdbdia,0,6)	! Dia com 6 bits
  oct1 = ISHFT(oct1,2)		! desloca 2 bits para a esquerda

  oct2 = IBITS(rdbtime(5),3,2)	! pega os 2 bits mais significativos de Hora
  ! Hora tem 5 bits, 3 vao para o proximo byte
  oct3 = oct1 + oct2		! completa o byte !!

  sec2(39:39) = char(oct3)       ! Guarda este byte na string sec2.

  oct1 = IBITS(rdbtime(5),0,3)	! agora os 3 bits restantes de Hora
  oct1 = ISHFT(oct1,5)		! desloca-os para a esquerda

  oct2 = IBITS(rdbtime(6),1,5)	! Minutos tem 6 bits. 5 ficam neste byte, e
  ! 1 sobra para o proximo. oct2 pega os 5 
  oct3 = oct1 + oct2		! completa o byte com Hora e Minuto

  sec2(40:40) = char(oct3)	! Guarda este byte na string sec2.

  oct1 = IBITS(rdbtime(6),0,1)	! Pega o menor bit de minuto
  oct1 = ISHFT(oct1,7)		! Com este deslocamento, o ultimo bit de
  ! minutos ja fica na posicao para merge.

  ! A Proxima informacao sao os Segundos (!), com 6 bits, mas para o nosso
  ! caso, pode ser sempre zero. (nao tenho segundos das PCDs.)

  ! A seguir, vem mais um bit com 0 (!) talvez apenas para completar o byte.
  ! Assim, podemos gravar o nosso byte.

  sec2(41:41) = char(oct1)	! Guarda este byte na string sec2.

  ! IDD - 6 bits
  ! oct1 = 0			! IHH - 5 bits
  ! oct2 = 0			! IMM - 6 bits
  ! oct3 = 0			! ISS - 6 bits
  ! (em buoy, tem mais 1 bit=0)
  !
  ! Guarda estes 3 bytes na string sec2
  ! sec2(39:41) = char(oct1)//char(oct2)//char(oct3)
  !
  sec2(42:42) = char(0) 
  sec2(43:43)= char(0)
  sec2(44:44)= char(0)
  valor = 1 !LAC - 1a. Parte da mensagem !LAC
  oct1 = IBITS(valor,0,1) !LAC 
  oct1 = ISHFT(oct1,1) !LAC 
  sec2(45:45)= char(oct1) 
  valor = 0 !LAC - Numero da correcao !LAC
  oct1 = IBITS(valor,0,6) !LAC 
  oct1 = ISHFT(oct1,2) !LAC 
  sec2(46:46)= char(oct1)       
  sec2(47:47)= char(0)
  sec2(48:48)= char(0)
  valor = 70 !LAC 
  oct1 = IBITS(valor,0,8) !LAC
  !oct1 = ISHFT(oct1,7) !LAC
  sec2(49:49)= char(oct1) ! Nivel de confianca !LAC
  ! sec2(49:49)= char(0)   
  sec2(50:50)= char(0)
  sec2(51:51)
= char(0)
  sec2(52:52)
= char(0)

  write (4, '(a)') sec2			! Grava a Secao 2

!
!		    PREPARA E GRAVA A SECAO 3
!

! Aqui eu preparo e gravo a lista de descritores. No momento, esta  lista
! nao esta completa, i.e. nem todos os dados que as pcds fornecem sao gra-
! vados.

! TAMANHO DA SECAO 3 : Esta secao possui 7 bytes de identificacao, mais a
! lista de descritores. Cada descritor usa 2 bytes, e como no momento uso
! 17 descritores, assim temos TAMANHO = 7+34 = 41 bytes). Esses valores /
! vao mudar, e o numero de descritores eh definido pela constante Ndescrit
! o Tamanho da secao 3 eh calculado no inicio desta rotina: Tam_sec3

sec3 = ''				! na duvida, inicializa...

sec3 = char(0)//char(0)//char(Tam_sec3)	! Bytes 1-3: tamanho da Secao 
					! com este comando, limitei
					! este tamanho em 256 bytes
					! cuidado aqui !!!

sec3(4:4) = char(0)			! byte reservado = 0

sec3(5:6) = char(0)//char(1)		! Numero de data subsets  (obser-
					! vacoes em cada registro  BUFR).
					! como o atual BDM so aceita BUFR
					! "single-section", esta  restri-
					! cao tambem se aplica aqui.

sec3(7:7) = char(128)			! Dados Observados (PCDs) e sem
					! compressao. Assim, o bit 1 = 1
					! e o bit 2 = 0. Os demais bits
					! sao reservados (= 0). em binario,
					! temos 128 = 10000000

 write (4, '(a)') sec3			! Grava a identificacao da secao 3

!
! PREPARA E GRAVA OS DESCRITORES BUFR 
!

! ** no momento nao tem o nome !! **
! descrit = char(1)//char(15)		! 0 01 015 - Station or site Name
! write (4, '(a)') descrit		! grava o descritor


 descrit = char(4)//char(1)		! 0 04 001 - Ano
 write (4, '(a)') descrit		! grava o descritor

 descrit = char(4)//char(2)		! 0 04 002 - Mes
 write (4, '(a)') descrit		! grava o descritor
 
 descrit = char(4)//char(3)		! 0 04 003 - Dia
 write (4, '(a)') descrit		! grava o descritor
 
 descrit = char(4)//char(4) 		! 0 04 004 - Hora
 write (4, '(a)') descrit		! grava o descritor

 descrit = char(4)//char(5)		! 0 04 005 - Minutos
 write (4, '(a)') descrit		! grava o descritor

 descrit = char(5)//char(1)		! 0 05 001 - Latitude(high accuracy)
 write (4, '(a)') descrit		! grava o descritor
 
 descrit = char(6)//char(1)		! 0 06 001 - Longitude(high accuracy)
 write (4, '(a)') descrit		! grava o descritor
 
 descrit = char(7)//char(1)		! 0 07 001 - station's height
 write (4, '(a)') descrit		! grava o descritor
 
 descrit = char(10)//char(4) 		! 0 10 004 - Pressao
 write (4, '(a)') descrit		! grava o descritor
 
 descrit = char(11)//char(11)		! 0 11 011 - wind direction (at 10M)
 write (4, '(a)') descrit		! grava o descritor

 descrit = char(11)//char(12)		! 0 11 012 - wind Speed (at 10M)
 write (4, '(a)') descrit		! grava o descritor

 descrit = char(12)//char(1)		! 0 12 001 - temperatura dry-bulb
 write (4, '(a)') descrit		! grava o descritor
 
 descrit = char(12)//char(16)		! 0 12 016 - temp. max. (24h a 2m)
 write (4, '(a)') descrit		! grava o descritor

 descrit = char(12)//char(17)		! 0 12 017 - temp. min. (24h a 2m)
 write (4, '(a)') descrit		! grava o descritor

 descrit = char(13)//char(3)		! 0 13 003 - Relative Humidity
 write (4, '(a)') descrit		! grava o descritor

 descrit = char(13)//char(20)		! 0 13 020 - Total precipitation past 3 hours
 write (4, '(a)') descrit		! grava o descritor
 
 descrit = char(10)//char(51)		! 0 13 051 - Pressure reduced to mean sea level
 write (4, '(a)') descrit		! grava o descritor
 
 oct1=IBITS(1,0,2)
 oct1=ISHFT(oct1,6)
 oct2=IBITS(1,0,6)
 oct3=oct1+oct2
 descrit = char(oct3)//char(17)	        ! 1 01 017 - Replicador 17 vezes
 write (4, '(a)') descrit		! grava o descritor

 descrit = char(33)//char(7)		! 0 33 007 - Confidence Level
 write (4, '(a)') descrit		! grava o descritor

!
!		    PREPARA E GRAVA A SECAO 4
!

! O Tamanho da Secao 4 fica nos bytes 1-3, o byte 4 e reservado (=0),
! e os dados comecam no byte 5...

! Assim, o tamanho da secao 4 sera dada por 4 bytes + os bits necessarios
! para todos os dados. para o nosso caso de 17 descritores, temos:
!
! ** no momento nao tem nome !!
! 0 01 015 	Station or site Name 		160 bits 
!
! 0 04 001 	Ano				12 bits
! 0 04 002 	Mes				 4 bits
! 0 04 003 	Dia				 6 bits
! 0 04 004 	Hora				 5 bits
! 0 04 005	Minutos				 6 bits
! 0 05 001 	Latitude(high accuracy)		25 bits
! 0 06 001	Longitude( " )			26 bits
! 0 07 001	station's height 		15 bits
! 0 10 004	Pressao				14 bits
! 0 11 011	wind direction (at 10M)		 9 bits
! 0 11 012	wind Speed (at 10M) 		12 bits
! 0 12 001	temperatura dry-bulb 		12 bits
! 0 12 016	temp. max. (24h a 2m) 		12 bits
! 0 12 017	temp. min. (24h a 2m) 		12 bits
! 0 13 003	Relative Humidity		 7 bits
! 0 13 020      Tot prec past 3 hours           14 bits
! 0 10 051      Press. red. to msl              14 bits
! 1 01 017      Replicator ----------------------------
! 0 33 007      Confidence Level            17x  7 bits
!
!				       TOTAL = 324 bits
!
! 	324 bits = 40 bytes + 4 bits. Este ultimo deve ser completado
! com 0s, de forma que a secao 4 tera = 4+40+1 = 45 bytes. (valor de-
! finido no inicio desta rotina - uma constante) Detalhe: O numero /
! de bytes desta secao deve ser sempre par !! 
!
! Tamanho = 4+40+1 = 45 => 46 bytes

!
! Bytes 1-3: tamanho da Secao. 
! Para o caso em que tam<255, so o 3o byte importa.
!

 sec4 = ''					! inicializa
 
 sec4 = char(0)//char(0)//char(Tam_sec4)	! Tamanho da secao

 sec4(4:4) = char(0)				! byte reservado (= 0)

!
!			PREPARA E GRAVA OS DADOS.
!

!
! Pedaco commentado, pois no momento nao gravo o nome !!
!
! AUX = '                    '	! inicializo...

! write(AUX,*) IdPCD%PCDCOD	! Preparo o Nome da PCD, no caso, serah ape-
 				! nas o codigo, que vem no nome do arquivo.
! AUX=adjustl(AUX)		! Aqui eu facco uma implementacao da funcao 
 				! STRING$ do BASIC ! i.e. converto o numero 
 				! em string

! sec4(5:24) = AUX	 	! Station Name, 160 bits=20 bytes, de 5 a 24
 				
! ---------------------------------------------------------------------------
! Agora sim a parte complicada, concatenar a sequencia de bits formando um
! "stream" continuo para ser gravado. As duas primeiras informacoes sao: O
! Ano com 12 bits, e o Mes com 4 bits.
! ---------------------------------------------------------------------------

 oct1 = IBITS(Hobs%ANO,4,8)	! Este eh o primeiro byte, com os 8 primeiros
 				! bits de informacao de ANO
 				
 				
 oct2 = IBITS(Hobs%ANO,0,4)	! Este recebe os 4 bits restantes de ANO, mas
 				! agora falta concatenar o proximo dado neste
 				! byte, antes de gravar. Entao:
 				
 oct2 = ISHFT(oct2,4)		! desloca esses 4 bits para a esquerda, abrin-
 				! do espaco para colocar os bits do proximo /
 				! dado (mes=4bits)
 				
 oct3 = IBITS(Hobs%MES,0,4)	! Os 4 bits significativos de MES vao p/ oct3

 oct2 = oct2 + oct3		! faz o merge dos bits de ano e mes. Agora 
 				! temos dois bytes completos, em oct1 e oct2,
 				! com 12 bits de ano e 4 bits de mes

 sec4(5:6) = char(oct1)//char(oct2)	! Guarda estes bytes na string sec4.



 oct1 = IBITS(Hobs%Dia,0,6)	! Dia com 6 bits
 oct1 = ISHFT(oct1,2)		! desloca 2 bits para a esquerda
 oct2 = IBITS(Hobs%HORA,3,2)	! pega os 2 bits mais significativos de Hora
 				! Hora tem 5 bits, 3 vao para o proximo byte
 oct3 = oct1 + oct2		! completa o byte !!
 sec4(7:7) = char(oct3)		! Guarda este byte na string sec4.

 oct1 = IBITS(Hobs%HORA,0,3)	! agora os 3 bits restantes de Hora
 oct1 = ISHFT(oct1,5)		! desloca-os para a esquerda

 oct2 = IBITS(Hobs%MIN,1,5)	! Minutos tem 6 bits. 5 ficam neste byte, e
 				! 1 sobra para o proximo. oct2 pega os 5 
 oct3 = oct1 + oct2		! completa o byte com Hora e Minuto
 sec4(8:8) = char(oct3)		! Guarda este byte na string sec4.

 oct1 = IBITS(Hobs%MIN,0,1)	! Pega o menor bit de minuto
 oct1 = ISHFT(oct1,7)		! Com este deslocamento, o ultimo bit de
 				! minutos ja fica na posicao para merge.

! ---------------------------------------------------------------------------
! Agora a latitude, aqui complica pela necessidade de ajustar a escala e
! referencia. A escala eh 5 e referencia -9000000, em 25 bits
! ---------------------------------------------------------------------------
 
 Valor = (IdPCD%Lat*10**5)-(-9000000)

 oct2 = IBITS(Valor,18,7)	! Pega os 7 bits mais significativos da
 				! latitude .
 oct3 = oct1 + oct2		! completa o byte com Minuto e latitude
 sec4(9:9) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,10,8)	! Proximos 8 bits de Latitude
 sec4(10:10) = char(oct1)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,2,8)	! Proximos 8 bits de Latitude
 sec4(11:11) = char(oct1)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,2)	! Pega os 2 menores bits da latitude
 oct1 = ISHFT(oct1,6)		! Com este deslocamento, os dois ultimos
 				! bits da latitude ja ficam para merge.

! ---------------------------------------------------------------------------
! Longitude, novamente escala e referencia, agora em 26 bits. 
! ---------------------------------------------------------------------------

 Valor = (IdPCD%Lon*10**5)-(-18000000)

 oct2 = IBITS(Valor,20,6)	! Pega os 6 bits mais significativos da
 				! longitude .

 oct3 = oct1 + oct2		! completa o byte com lat. e longitude
 sec4(12:12) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,12,8)	! Proximos 8 bits da Longitude (12-19)
 sec4(13:13) = char(oct1)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,4,8)	! Proximos 8 bits da Longitude (4-11)
 sec4(14:14) = char(oct1)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,4)	! Pega os 4 menores bits da long. (0-3)
 oct1 = ISHFT(oct1,4)		! Deslocamento para merge.

! ---------------------------------------------------------------------------
! 0 07 001	station's height 	meter	0	-400	15 bits
!
! Agora a escala eh =1 , e soma-se 400 na referencia (p/ os casos de estacoes
! que ficam abaixo do nivel do mar)
! ---------------------------------------------------------------------------

 valor = (IdPCD%Ha)+400
 oct2 = IBITS(valor,11,4)	! Pega os 4 maiores bits da altitude (11-14)
 oct3 = oct1 + oct2		! junta os byte com logitude e altitude
 sec4(15:15) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,3,8)	! Pega o byte completo da altitude (3-10)
 sec4(16:16) = char(oct1)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,3)	! Pega os 3 menores bits da altitude (0-2)
 oct1 = ISHFT(oct1,5)		! Deslocamento para merge.

! ---------------------------------------------------------------------------
! 0 10 004	Pressao			Pascal	-1	0	14 bits
!
! Como os dados das pcds sao fornecidos em mb, preciso converte-lo para
! Pascal (1 mb = 100 Pa)
! ---------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==010004) then
 		medida = Obs(i)%valor *100	! Converte para Pascal
 		valor = INT( medida/10 )	! Acerta a escala da Pressao
 	end if 
 End do

 oct2 = IBITS(valor,9,5)	! Pega os 5 maiores bits da Pressao (9-13)
 oct3 = oct1 + oct2		! junta os byte com pressao e altitude
 sec4(17:17) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,1,8)	! Pega o byte completo da Pressao (1-8)
 sec4(18:18) = char(oct1)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,1)	! Pega o ultimo bit da pressao (0-0)
 oct1 = ISHFT(oct1,7)		! Deslocamento para merge.

! -------------------------------------------------------------------------
! 0 11 011	wind direction (at 10M)	degree	0	0	9 bits
! -------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==011011) then
 		medida = Obs(i)%valor
 		valor = INT(medida)	! Aqui nao ha escala nem referencia
 	end if 
 End do

 oct2 = IBITS(valor,2,7)	! Pega os 7 maiores bits (2-8)
 oct3 = oct1 + oct2		! junta os byte com pressao e dir.vento
 sec4(19:19) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,2)	! Pega os 2 ultimos bits da dir.vento (0-1)
 oct1 = ISHFT(oct1,6)		! Deslocamento para merge.

! --------------------------------------------------------------------------
! 0 11 012	wind Speed (at 10M) 	m s-1	1	0	12 bits
! --------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==011012) then
 		medida = Obs(i)%valor
 		valor = INT(medida*10**(1))	! Acerta a escala vel. vento
 	end if 
 End do

 oct2 = IBITS(valor,6,6)	! Pega os 6 maiores bits de wind speed (6-11)
 oct3 = oct1 + oct2		! junta os byte com dir. e vel. vento
 sec4(20:20) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,6)	! Pega os ultimos 6 bits da vel. (0-5)
 oct1 = ISHFT(oct1,2)		! Deslocamento para merge.

! ---------------------------------------------------------------------------
! 0 12 001	temperatura dry-bulb 	K	1	0	12 bits
!
! A temperatura das PCDs vem em Celsius, Portanto preciso converter p/ K
! ---------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==012001) then
 		medida = Obs(i)%valor + 273.5	! converte para Kelvin
 		valor = INT(medida*10**(1))	! Acerta a escala da Temperat.
 	end if 
 End do

 oct2 = IBITS(valor,10,2)	! Pega os 2 maiores bits da temperat. (10-11)
 oct3 = oct1 + oct2		! junta com os bits da vel. vento  
 sec4(21:21) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,2,8)	! Pega o byte completo da Temperat. (2-9)
 sec4(22:22) = char(oct1)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,2)	! Pega os 2 ultimos bits da temperat. (0-1)
 oct1 = ISHFT(oct1,6)		! Deslocamento para merge.

! ---------------------------------------------------------------------------
! 0 12 016	temp. max. (24h a 2m) 	K	1	0	12 bits
!
! A temperatura das PCDs vem em Celsius, Portanto preciso converter p/ K
! ---------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==012016) then
 		medida = Obs(i)%valor + 273.5	! converte para Kelvin
	 	valor = INT(medida*10**(1))	! Acerta escala da Temp.max.
 	end if 
 End do

 oct2 = IBITS(valor,6,6)	! Pega os 6 maiores bits da t.max. (6-11)
 oct3 = oct1 + oct2		! junta com os bits da temperatura
 sec4(23:23) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,6)	! Pega os ultimos 6 bits da vel. (0-5)
 oct1 = ISHFT(oct1,2)		! Deslocamento para merge (2 bits)

! -------------------------------------------------------------------------
! 0 12 017	temp. min. (24h a 2m) 	K	1	0	12 bits
!
! A temperatura das PCDs vem em Celsius, Portanto preciso converter p/ K
! -------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==012017) then
 		medida = Obs(i)%valor + 273.5	! converte para Kelvin
 		valor = INT(medida*10**(1))	! Acerta escala da Temp.Min.
 	end if 
 End do

 oct2 = IBITS(valor,10,2)	! Pega os 2 maiores bits da temperat. (10-11)
 oct3 = oct1 + oct2		! junta com os bits da temperat. Max.
 sec4(24:24) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,2,8)	! Pega o byte completo da Temp. Minima (2-9)
 sec4(25:25) = char(oct1)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,2)	! Pega os 2 ultimos bits da temp. Min. (0-1)
 oct1 = ISHFT(oct1,6)		! Deslocamento para merge.

! --------------------------------------------------------------------------
! 0 13 003	Relative Humidity	%	0	0	7 bits
! --------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==013003) then
 		medida = Obs(i)%valor
 		valor = INT(medida)	! Nao eh preciso acertar escala 
 	end if 
 End do

 oct2 = IBITS(valor,1,6)	! Pega os 6 maiores bits da umidade (1-6)
 oct3 = oct1 + oct2		! junta com os bits da temperat. Max.
 sec4(26:26) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,1)	! Pega o ultimo bit (0-0)
 oct1 = ISHFT(oct1,7)		! Deslocamento para ficar no canto

! --------------------------------------------------------------------------
! 0 13 020      Tot prec past 3 hours   kg m-2  1      -1       14 bits
! --------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==013020) then
           medida = (Obs(i)%valor*10**(1)) - (-1)
           valor = INT(medida)
           !print *,Obs(i)%valor, medida, valor 
 	end if 
 End do

 oct2 = IBITS(valor,7,7)	! Pega os 7 maiores bits da precipitao (7-13)
 oct3 = oct1 + oct2		! junta com o bit da umidade relativa
 sec4(27:27) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,7)	! Pega os 7 bits restantes da precipitao (1-7)
 oct1 = ISHFT(oct1,1)		! Deslocamento para ficar no canto

! --------------------------------------------------------------------------
! 0 10 051      Press. red. to msl      Pa      -1      0     14 bits
! --------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==010051) then
           medida = Obs(i)%valor *100	! Converte para Pascal
           valor = INT( medida/10 ) 
 	end if 
 End do

 oct2 = IBITS(valor,13,1)	! Pega o maior bit da press reduzida (13-13)
 oct3 = oct1 + oct2		! junta com os bits da precipitacao
 sec4(28:28) = char(oct3)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,5,8)	! Pega os 8 bits seguinte press reduzida (5-12)
 sec4(29:29) = char(oct1)	! Guarda este byte na string sec4.

 oct1 = IBITS(valor,0,5)	! Pega os 5 bits restantes da precipitao (0-4)
 oct1 = ISHFT(oct1,3)		! Deslocamento para ficar no canto
 

! ---------------------------------------------------------------------------
! Colocando o Controle de qualidade - 0 33 007   %                7 Bits
! ---------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==004001) then
           valor = Obs(i)%QC
        end if
 End do

 oct2 = IBITS(valor,4,3)	! Pega 3 bits do QC (7bits)
 oct3 = oct1 + oct2		! junta com os bits da Presso reduzida
 sec4(30:30) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------
 
 oct1 = IBITS(valor,0,4)	! Pega 4 bits da QC
 oct1 = ISHFT(oct1,4)		

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==004002) then
           valor = Obs(i)%QC
 	end if 
 End do

 oct2 = IBITS(valor,3,4)	! Pega 4 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(31:31) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,3)	! Pega 3 bits da QC
 oct1 = ISHFT(oct1,5)		

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==004003) then
           valor = Obs(i)%QC
 	end if 
 End do

 oct2 = IBITS(valor,2,5)	! Pega 5 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(32:32) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,2)	! Pega 2 bits da QC
 oct1 = ISHFT(oct1,6)		

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==004004) then
           valor = Obs(i)%QC
  	end if 
 End do

 oct2 = IBITS(valor,1,6)	! Pega 6 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(33:33) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,1)	! Pega 1 bit da QC
 oct1 = ISHFT(oct1,7)		
 
 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==004005) then
           valor = Obs(i)%QC
  	end if 
 End do

 oct2 = IBITS(valor,0,7)	! Pega 7 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(34:34) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==005001) then
           valor = Obs(i)%QC
 	end if 
 End do

 oct1 = IBITS(valor,0,7)	! Pega 7 bits da QC
 oct1 = ISHFT(oct1,1)

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==006001) then
           valor = Obs(i)%QC
        end if
 End do

 oct2 = IBITS(valor,6,1)	! Pega 1 bit da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(35:35) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,6)	! Pega 6 bits da QC
 oct1 = ISHFT(oct1,2)

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==007001) then
           valor = Obs(i)%QC
        end if
 End do

 oct2 = IBITS(valor,5,2)	! Pega 2 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(36:36) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,5)	! Pega 5 bits da QC
 oct1 = ISHFT(oct1,3)

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==010004) then
           valor = Obs(i)%QC 
 	end if 
 End do

 oct2 = IBITS(valor,4,3)	! Pega 3 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(37:37) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,4)	! Pega 4 bits da QC
 oct1 = ISHFT(oct1,4)

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==011011) then
           valor = Obs(i)%QC 
 	end if 
 End do

 oct2 = IBITS(valor,3,4)	! Pega 4 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(38:38) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,3)	! Pega 3 bits da QC
 oct1 = ISHFT(oct1,5)

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==011012) then
           valor = Obs(i)%QC
 	end if 
 End do

 oct2 = IBITS(valor,2,5)	! Pega 5 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(39:39) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,2)	! Pega 2 bits da QC
 oct1 = ISHFT(oct1,6)

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==012001) then
           valor = Obs(i)%QC 
 	end if 
 End do

 oct2 = IBITS(valor,1,6)	! Pega 6 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(40:40) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,1)	! Pega 1 bits da QC
 oct1 = ISHFT(oct1,7)

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==012016) then
           valor = Obs(i)%QC 
 	end if 
 End do

 oct2 = IBITS(valor,0,7)	! Pega 7 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(41:41) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==012017) then
           valor = Obs(i)%QC 
 	end if 
 End do

 oct1 = IBITS(valor,0,7)	! Pega 7 bits da QC
 oct1 = ISHFT(oct1,1)

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==013003) then
           valor = Obs(i)%QC 
 	end if 
 End do

 oct2 = IBITS(valor,6,1)	! Pega 1 bit da QC
 oct3 = oct1 + oct2
 sec4(42:42) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,6)	! Pega 6 bits da QC
 oct1 = ISHFT(oct1,2)

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==013020) then
           valor = Obs(i)%QC 
 	end if 
 End do

 oct2 = IBITS(valor,5,2)	! Pega 2 bit da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(43:43) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------

 oct1 = IBITS(valor,0,5)	! Pega 5 bits da QC
 oct1 = ISHFT(oct1,3)

 valor = MISSING 		! Missing Value = todos os bits=1s !
 Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==010051) then
           valor = Obs(i)%QC
 	end if 
 End do

 oct2 = IBITS(valor,4,3)	! Pega 3 bits da QC
 oct3 = oct1 + oct2		! junta com os bits da QC anterior
 sec4(44:44) = char(oct3)	! Guarda este byte na string sec4.

! ---------------------------------------------------------------------------
  
 oct1 = IBITS(valor,0,4)	! Pega 4 bits da QC
 oct1 = ISHFT(oct1,4)
 sec4(45:45) = char(oct1)

! ---------------------------------------------------------------------------

 sec4(46:46) = char(0)

 write (4, '(a)') sec4		! grava a string com a secao 4 inteira.

!
!		    PREPARA E GRAVA A SECAO 5
!

 sec5 = '7777'
 write (4, '(a)') sec5

!
!		     FECHA ARQUIVO E RETORNA
!
 
 close(4)

 RETURN
 
 END 


!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
!****************************************************************************
!
 SUBROUTINE GRAVABUFR2(IdPCD,Obs,HObs,NObs,Table_B,ct)
   !
   USE MREADPCD
   !
   type Bufr_Table
      integer:: f,x,y,scale,ref,nbits
      character(len=46) :: BUFR_desc
      character(len=13) :: Unit
   end type Bufr_Table

   !
   !		 DEFINICOES DAS VARIAVEIS DE ENTRADA
   !
   !
   type(MREADPCD_Id), intent(in):: IdPCD	! Identificacao de uma 
   ! PCD (Lat,Lon,etc..)
   type(MREADPCD_Date), intent(in):: HObs ! Data e Hora de uma 
   ! observacao de PCD 
   type(MREADPCD_Obs),dimension(29), intent(in)::Obs ! Dados Observacionais 

   integer,  intent(in):: Nobs ! Numero de Dados em uma observacoes

   type(Bufr_Table),dimension(1000), intent(in):: Table_B  ! Tabela B BUFR

   INTEGER,INTENT(IN)::ct                                 ! Tabela B BUFR

   !
   !		  DEFINICOES DE VARIAVEIS LOCAIS
   !
   !
   character (1)  sec     ! Para gravar 1 byte
   character (8)  sec0	! Para gravar a secao 0 (8 bytes)
   character (18) sec1	! Para gravar os 18 bytes da secao 1
   character (52) sec2 	! Nesta versao a secao 2 eh uma copia do
   ! mdb_pcd.f, e cheia de valores arbitrados.
   character (7) sec3	! A identificacao da secao 3 tem 7 bytes
   character (2) descrit	! Os descritores sempre tem 2 bytes cada ! 
   character (28) sec4	! No nosso caso, a secao 4 tem 28 bytes. ! *CUIDADO*
   ! Melhor aqui eh gravar byte a byte -> mudar!
   character (4)  sec5	! Para gravar o final do BUFR = '7777'
   !
   INTEGER Tam_BUFR	! Tamanho total do arquivo BUFR (grav. na secao 1)
   INTEGER Tam_sec2	! Numero de bytes da secao 2
   INTEGER Tam_sec3	! Numero de bytes da secao 3
   INTEGER Tam_sec4	! Numero de bytes da secao 4
   ! 
   INTEGER Ndescrit	! Numero de descritores utilizados no momento.
   ! 
   INTEGER(kind=1) oct1	! octetos (os bytes) que serao gravados
   INTEGER(kind=1) oct2	! no stream binario. Varios usos.
   INTEGER(kind=1) oct3
   !
   INTEGER(kind=4) MISSING ! missing value em BUFR, todos bits=1
   REAL (kind=4) medida	! Para receber os valores lidos das PCDs
   INTEGER(kind=4) valor	! para diversos usos
   CHARACTER (20) AUX	! String auxiliar para diversos usos
   INTEGER i,j,k,l	! Para os loops.
   !
   INTEGER fxy            ! Table B Bufr
   INTEGER NTOTBITS, BYTESIMPAR, NTOTBYTES
   INTEGER(kind=4), DIMENSION (1000) :: values
   INTEGER, DIMENSION (1000) :: NRESTABITS, NBITSOBS, NBYTESTOPUT
   !
   !
   !
   !		 CALCULA O TAMANHO DO ARQUIVO BUFR
   !

   MISSING = HUGE(MISSING)

   Ndescrit = 15					! No momento, uso 15 descritores
   ! apenas

   Tam_sec2 = 52					! Numero de bytes da secao 2
   Tam_sec3 = 7 + (2*Ndescrit)			! Tamanho secao 3 - descritores
   Tam_sec4 = 4+23+1				! Tamanho da secao 4 para estes
   ! descritores. 

   Tam_BUFR = 8 + 18 + 4 				! tamanho Secao 0, 1 e 5
   Tam_BUFR = Tam_BUFR + Tam_sec2
   Tam_BUFR = Tam_BUFR + Tam_sec3			! Tamanho total do registro
   Tam_BUFR = Tam_BUFR + Tam_sec4			! Tamanho secao 4 - os dados.

   !
   !		ABRE ARQUIVO DE SAIDA (PARA APPEND)
   !

   open(4,file='pcd.rdb',ORGANIZATION='SEQUENTIAL',POSITION='APPEND',CARRIAGECONTROL='NONE')

   !
   !		        PREPARA E GRAVA A SECAO 0
   !

   ! A variavel Tam_BUFR eh integer*4, sendo que o primeiro bit sera sempre 0,
   ! pois ela devera sempre ser um numero positivo. Como o tamanho do arquivo
   ! BUFR eh registrado em 3 bytes, o tamanho maximo do BUFR sera de 23 bits,
   ! igual a 8388607 bytes.
   !
   ! A questao aqui eh extrair o valor numerico destes bytes para guarda-los
   ! nas posicoes de 5-7, na secao 0. Eh preciso eliminar o byte mais signifi-
   ! cativo da variavel Tam_BUFR, que alias devera ser sempre = 0 ! - Qual o
   ! valor entao, dos bytes 2, 3 e 4 ?
   !
   ! Eh preciso cuidado aqui, por causa do famoso problema do BigEndianXlittle
   ! Se o valor dos bytes for acessado diretamente da memoria, teremos este
   ! problema. Formas alternativas de tratar isto sao : usar mascaras de bits,
   ! operacoes matematicas ou deslocamento de bits. Uma grande vantagem do F90
   ! eh a conveniente funcao IBITS !!! :-D

   oct3 = IBITS(Tam_BUFR,0,8)			! o menor byte do tamanho
   oct2 = IBITS(Tam_BUFR,8,8)			! o byte do meio
   oct1 = IBITS(Tam_BUFR,16,8)			! o maior byte do tamanho

   sec0 = ''					! inicializa string de saida
   sec0 = 'BUFR'					! o indicador
   sec0(5:7) = char(oct1)//char(oct2)//char(oct3)	! Tamanho total do arquivo
   sec0(8:8) = char(2)				! BUFR edicao 2 

   write (4, '(a)') sec0 				! Grava secao 0

   !
   !		    PREPARA E GRAVA A SECAO 1
   !

   sec1 = ''				! inicializa

   sec1 = char(0)//char(0)//char(18)	! Bytes 1 a 3 contem o tamanho da 
   ! secao 1. Aqui eh sempre = 18 !

   sec1(4:4) = char(0)			! Byte 4 contem a versao da BUFR 
   ! master table. Para meteorologia=0

   sec1(5:6) = char(0)//char(46) 		! Os bytes 5 e 6 contem o codigo do 
   ! centro que gerou o arquivo  BUFR,
   ! segundo a code table 0 01 031 (ver 
   ! manual WMO). No caso, o codigo do 
   ! INPE (Brazilian Space Agency !?) 
   ! eh 46 (octeto 6)

   sec1(7:7) = char(0)			! Update sequence number = 0

   sec1(8:8) = char(128) 			! Bit 1=0 (nao ha secao 2)
   ! os demais bits sao reservados.

   sec1(9:9) = char(0)			! Data category type (BUFR Table A)
   ! 0 corresponde a surface data/land

   sec1(10:10) = char(7)			! Data category sub-type
   ! Por enquanto, 7 para as PCDs, se-
   ! gundo sugestao da  Rosangela  (O 
   ! BDM usa este valor para classifi-
   ! car o dado ?)

   sec1(11:11) = char(2)			! Versao utilizada da tabela Master/WMO

   sec1(12:12) = char(1)			! Versao utilizada da tabela local/INPE

   ! Um detalhe importante nos proximos bytes (13-18), e o fato de que centro dife-
   ! rentes estao usando padroes diferentes aqui !!! - A ordem a seguir eh a reco -
   ! mendada pela WMO, mas o NCEP, por exemplo, poe o ano nos bytes 13 e 14, fazen-
   ! do um shift nos demais dados e o minuto no byte 18. 
   ! Devido a estas diferencas, uma boa politica pode ser: nao usar a data que es -
   ! tiver nesta secao, ou decodificar de acordo com o centro que gerou a mensagem.

   valor = Hobs%ANO - 2000
   sec1(13:13) = char(Valor)		! Ano do Seculo

   Valor = Hobs%MES
   sec1(14:14) = char(Valor)		! Mes da Observacao

   Valor = Hobs%Dia
   sec1(15:15) = char(Valor)		! Dia da Observacao

   sec1(16:16) = char(Hobs%HORA)		! Hora da Observacao

   sec1(17:17) = char(Hobs%MIN)		! Minuto da Observacao

   sec1(18:18) = char(0)			! Reservado para uso local - Nao uso !

   write (4, '(a)') sec1			! Grava a Secao 1 


   !
   !		    PREPARA E GRAVA A SECAO 2
   !

   sec2 = ''				! inicializa

   sec2 = char(0)//char(0)//char(Tam_sec2) ! Bytes 1 a 3 contem o tamanho

   sec2(4:4) = char(0)			! Byte 4 eh reservado (=0)

   sec2(5:5) = char(0)			! Ntype com 8 bits. No nosso caso, 0
   ! para estacao superficie.

   sec2(6:6) = char(7)			! NSBtype com 8 bits. sugestao da ro:
   ! pcd = 7 

   ! ---------------------------------------------------------------------------
   ! Agora sim a parte complicada, concatenar a sequencia de bits formando um
   ! "stream" continuo para ser gravado. 
   ! --> Puxa ! uma boa parte disto e "copy & paste" da secao 4 !
   ! ---------------------------------------------------------------------------

   oct1 = IBITS(Hobs%ANO,4,8)	! Este eh o primeiro byte, com os 8 primeiros
   ! bits de informacao de ANO


   oct2 = IBITS(Hobs%ANO,0,4)	! Este recebe os 4 bits restantes de ANO, mas
   ! agora falta concatenar o proximo dado neste
   ! byte, antes de gravar. Entao:

   oct2 = ISHFT(oct2,4)		! desloca esses 4 bits para a esquerda, abrin-
   ! do espaco para colocar os bits do proximo /
   ! dado (mes=4bits)

   oct3 = IBITS(Hobs%MES,0,4)	! Os 4 bits significativos de MES vao p/ oct3

   oct2 = oct2 + oct3		! faz o merge dos bits de ano e mes. Agora 
   ! temos dois bytes completos, em oct1 e oct2,
   ! com 12 bits de ano e 4 bits de mes

   sec2(7:8) = char(oct1)//char(oct2)	! Guarda estes bytes na string sec2.


   oct1 = IBITS(Hobs%Dia,0,6)	! Dia com 6 bits
   oct1 = ISHFT(oct1,2)		! desloca 2 bits para a esquerda
   oct2 = IBITS(Hobs%HORA,3,2)	! pega os 2 bits mais significativos de Hora
   ! Hora tem 5 bits, 3 vao para o proximo byte
   oct3 = oct1 + oct2		! completa o byte !!

   sec2(9:9) = char(oct3)		! Guarda este byte na string sec2.

   oct1 = IBITS(Hobs%HORA,0,3)	! agora os 3 bits restantes de Hora
   oct1 = ISHFT(oct1,5)		! desloca-os para a esquerda

   oct2 = IBITS(Hobs%MIN,1,5)	! Minutos tem 6 bits. 5 ficam neste byte, e
   ! 1 sobra para o proximo. oct2 pega os 5 
   oct3 = oct1 + oct2		! completa o byte com Hora e Minuto

   sec2(10:10) = char(oct3)	! Guarda este byte na string sec2.

   oct1 = IBITS(Hobs%MIN,0,1)	! Pega o menor bit de minuto
   oct1 = ISHFT(oct1,7)		! Com este deslocamento, o ultimo bit de
   ! minutos ja fica na posicao para merge.

   ! A Proxima informacao sao os Segundos (!), com 6 bits, mas para o nosso
   ! caso, pode ser sempre zero. (nao tenho segundos das PCDs.)

   ! A seguir, vem mais um bit com 0 (!) talvez apenas para completar o byte.
   ! Assim, podemos gravar o nosso byte.

   sec2(11:11) = char(oct1)	! Guarda este byte na string sec2.

   ! Agora a vez das Longitudes e Latitudes. Nota: aqui eh primeiro a longitude!

   ! ---------------------------------------------------------------------------
   ! Longitude, com ajuste de escala e referencia, agora em 26 bits. 
   ! ---------------------------------------------------------------------------

   Valor = (IdPCD%Lon*10**5)-(-18000000)

   oct1 = IBITS(Valor,18,8)	! Pega os 8 bits mais significativos da
   ! longitude (18-25)

   oct2 = IBITS(valor,10,8)	! Proximos 8 bits da Longitude (10-17)

   oct3 = IBITS(valor,2,8)	! Proximos 8 bits da Longitude (2-9)

   sec2(12:14) = char(oct1)//char(oct2)//char(oct3)	! Guarda na sec2

   ! Sobraram 2 bits, e o Proximo valor a ser gravado eh: zero, com 6 bits !!
   ! portanto, basta gravar os 2 bits, apos deslocar 6 posicoes.

   oct1 = IBITS(valor,0,2)	! Ultimos 2 bits da Longitude (0-1)
   oct1 = ISHFT(oct1,6)		! Deslocamento para Incluir os 6 bits nulos.

   sec2(15:15) = char(oct1)	! Guarda este byte na string sec4.

   ! ---------------------------------------------------------------------------
   ! Latitude, com ajuste de escala e
   ! referencia. A escala eh 5 e referencia -9000000, em 25 bits
   ! ---------------------------------------------------------------------------

   Valor = (IdPCD%Lat*10**5)-(-9000000)

   oct1 = IBITS(Valor,17,8)	! Pega os 8 bits mais significativos da
   ! latitude (17-24)
   sec2(16:16) = char(oct1)	! Guarda este byte na string sec2

   oct1 = IBITS(valor,9,8)	! Proximos 8 bits de Latitude (9-16)
   sec2(17:17) = char(oct1)	! Guarda este byte na string sec2

   oct1 = IBITS(valor,1,8)	! Proximos 8 bits de Latitude (1-8)
   sec2(18:18) = char(oct1)	! Guarda este byte na string sec2

   oct1 = IBITS(valor,0,1)	! Pega o ultimo bit da latitude
   oct1 = ISHFT(oct1,7)		! Com este deslocamento, o ultimo
   ! bit da latitude fica pronto

   ! Como se adicionam 7 bits 0 (zero), posso gravar direto. 				

   sec2(19:19) = char(oct1)	! Guarda este byte na string sec2

   ! --------------------------------------------------------------------------
   ! Agora Gravo o Nome, em 5 bytes (do byte 20 ao 24)
   ! --------------------------------------------------------------------------

   AUX = '                  '

   write(AUX,*) IdPCD%PCDCOD	! Preparo o Nome da PCD, no caso, serah ape-
   ! nas o codigo, que vem no nome do arquivo.
   AUX=adjustl(AUX)		! Aqui eu facco uma implementacao da funcao 
   ! STRING$ do BASIC ! i.e. converto o numero 
   ! em string

   sec2(20:36) = AUX(1:5) 	! Station Number, com 5 bytes

   ! -------------------------------
   ! 2 Bytes com outra informacao... (qual ?)
   ! -------------------------------


   sec2(37:38) = char(0)//char(Tam_BUFR)	! ILenght - 16 bits Tamanho Bufr ?!

   ! --------------------------------------------------------------------------
   !  Agora a hora em que o boletim foi escrito no RDB (faz sentido no CPTEC?)
   ! --------------------------------------------------------------------------

   ! mais 3 bytes, temporariamente com zeros....



   oct1 = IBITS(Hobs%Dia,0,6)	! Dia com 6 bits
   oct1 = ISHFT(oct1,2)		! desloca 2 bits para a esquerda
   oct2 = IBITS(Hobs%HORA,3,2)	! pega os 2 bits mais significativos de Hora
   ! Hora tem 5 bits, 3 vao para o proximo byte
   oct3 = oct1 + oct2		! completa o byte !!

   sec2(39:39) = char(oct3)		! Guarda este byte na string sec2.

   oct1 = IBITS(Hobs%HORA,0,3)	! agora os 3 bits restantes de Hora
   oct1 = ISHFT(oct1,5)		! desloca-os para a esquerda

   oct2 = IBITS(Hobs%MIN,1,5)	! Minutos tem 6 bits. 5 ficam neste byte, e
   ! 1 sobra para o proximo. oct2 pega os 5 
   oct3 = oct1 + oct2		! completa o byte com Hora e Minuto

   sec2(40:40) = char(oct3)	! Guarda este byte na string sec2.

   oct1 = IBITS(Hobs%MIN,0,1)	! Pega o menor bit de minuto
   oct1 = ISHFT(oct1,7)		! Com este deslocamento, o ultimo bit de
   ! minutos ja fica na posicao para merge.

   ! A Proxima informacao sao os Segundos (!), com 6 bits, mas para o nosso
   ! caso, pode ser sempre zero. (nao tenho segundos das PCDs.)

   ! A seguir, vem mais um bit com 0 (!) talvez apenas para completar o byte.
   ! Assim, podemos gravar o nosso byte.

   sec2(41:41) = char(oct1)	! Guarda este byte na string sec2.



   ! IDD - 6 bits
   ! oct1 = 0			! IHH - 5 bits
   ! oct2 = 0			! IMM - 6 bits
   ! oct3 = 0			! ISS - 6 bits
   ! (em buoy, tem mais 1 bit=0)

   ! Guarda estes 3 bytes na string sec2

   ! sec2(39:41) = char(oct1)//char(oct2)//char(oct3)

   sec2(42:42) = char(0)
   sec2(43:43)= char(0)
   sec2(44:44)= char(0)
   !
   sec2(45:45)= char(0)
   ! valor = 1
   ! oct1 = IBITS(valor,0,1)
   ! oct1 = ISHFT(oct1,7)
   sec2(46:46)= char(0)  ! 1a. Parte da mensagem - LAC (??)      
   sec2(47:47)= char(0)
   sec2(48:48)= char(0)
   valor = 70
   oct1 = IBITS(valor,0,8)
   sec2(49:49)= char(oct1)  ! Nivel de coincidencia - LAC
   ! sec2(49:49)= char(0)   
   sec2(50:50)= char(0)
   sec2(51:51)
= char(0)
   sec2(52:52)
= char(0)

   write (4, '(a)') sec2			! Grava a Secao 2 

   !
   !		    PREPARA E GRAVA A SECAO 3
   !

   ! Aqui eu preparo e gravo a lista de descritores. No momento, esta  lista
   ! nao esta completa, i.e. nem todos os dados que as pcds fornecem sao gra-
   ! vados.

   ! TAMANHO DA SECAO 3 : Esta secao possui 7 bytes de identificacao, mais a
   ! lista de descritores. Cada descritor usa 2 bytes, e como no momento uso
   ! 17 descritores, assim temos TAMANHO = 7+34 = 41 bytes). Esses valores /
   ! vao mudar, e o numero de descritores eh definido pela constante Ndescrit
   ! o Tamanho da secao 3 eh calculado no inicio desta rotina: Tam_sec3
   !
   print *, 'Para testar fxy sec3'
   sec = ''
   NTOTBITS=Nobs*2*8
   NTOTBYTES=NTOTBITS/8
   !
   print *, ' NTOTBI ',  NTOTBITS, '/8= ', NTOTBYTES, ' RES',NTOTBITS-NTOTBYTES*8
   !
   NTOTBYTES=NTOTBYTES+INT(NTOTBITS-NTOTBYTES*8)+ 7 ! 7 Bytes de identificao
   print *, ' NTOTBY ', NTOTBYTES
   ! 
   Tam_sec3=NTOTBYTES
   !
   !
   oct1 = IBITS(Tam_sec3,0,8)
   sec = char(oct1)
   write (4, '(a)') sec
   oct2 = IBITS(Tam_sec3,8,8)
   sec = char(oct1)
   write (4, '(a)') sec
   oct3 = IBITS(Tam_sec3,16,8)
   sec = char(oct1)
   write (4, '(a)') sec
   !
   sec = char(0) ! Byte Reservado
   write (4, '(a)') sec 
   !
   sec = char(0)
   write (4, '(a)') sec
   !
   sec = char(1)
   write (4, '(a)') sec
   !
   sec = char(128) ! 128 = 10000000 - Dados Observados (PCDs) e sem compressao
   write (4, '(a)') sec
   !
   ! PREPARA E GRAVA OS DESCRITORES BUFR 
   !
   Do i=1,Nobs
      DO j=1,ct
         fxy=Table_B(j)%f*100000+Table_B(j)%x*1000+Table_B(j)%y 
         if (Obs(i)%BUFRCOD==fxy) then
            descrit = char(Table_B(j)%f)
            write (4, '(a)') descrit
            descrit = char(Table_B(j)%x)
            write (4, '(a)') descrit
            descrit = char(Table_B(j)%y)
            write (4, '(a)') descrit
         END IF
      END DO
   END DO
   !
   !		    PREPARA E GRAVA A SECAO 4
   !

   ! O Tamanho da Secao 4 fica nos bytes 1-3, o byte 4 e reservado (=0),
   ! e os dados comecam no byte 5...

   ! Assim, o tamanho da secao 4 sera dada por 4 bytes + os bits necessarios
   ! para todos os dados. 
   !
   !************************************************************************
   !
   ! *    **** ****
   ! *    *  * *
   ! *    **** *
   ! *    *  * *
   ! **** *  * ****
   !
   !************************************************************************
   !
   !
   print *, 'Para testar fxy sec4'
   sec = ''
   !
   NTOTBYTES=0
   NTOTBITS=0
   !
   DO i=0, 1000
      NRESTABITS(i)=0
      NBITSOBS(i)=0
      NBYTESTOPUT(i)=0
   ENDDO
   Do i=1,Nobs
      DO j=1,ct
         fxy=Table_B(j)%f*100000+Table_B(j)%x*1000+Table_B(j)%y 
         if (Obs(i)%BUFRCOD==fxy) then
            values(i) = INT((Obs(i)%valor)*10**Table_B(j)%scale) - Table_B(j)%ref !
            NTOTBITS=Table_B(j)%NBITS+NTOTBITS ! Para contar bits lidos
            !
            NRESTABITS(i)=MOD(Table_B(j)%NBITS,8)
            !
            NBYTESTOPUT(i)= INT(Table_B(j)%NBITS/8)
            !
            PRINT *, fxy, values(i), Table_B(j)%NBITS, NBYTESTOPUT(i),  NRESTABITS(i)
         END IF
      END DO
   END DO
   NTOTBYTES=NTOTBITS/8
   !
   print *, ' NTOTBI ',  NTOTBITS, '/8= ', NTOTBYTES, ' RES ',NTOTBITS-NTOTBYTES*8
   !
   NTOTBYTES=NTOTBYTES+INT(NTOTBITS-NTOTBYTES*8)+ 7 ! 7 Bytes de identificao
   BYTESIMPAR=MOD(NTOTBYTES,2)
   print *, ' NTOTBY ', NTOTBYTES, ' IMPAR ', BYTESIMPAR
   IF (BYTESIMPAR==1) NTOTBYTES=NTOTBYTES+1
   print *, ' NTOTBY ', NTOTBYTES
   ! 
   Tam_sec4=NTOTBYTES
   ! 
   !
   oct1 = IBITS(Tam_sec4,0,8)
   sec = char(oct1)
   write (4, '(a)') sec
   oct1 = IBITS(Tam_sec4,8,8)
   sec = char(oct1)
   write (4, '(a)') sec
   oct1 = IBITS(Tam_sec4,16,8)
   sec = char(oct1)
   write (4, '(a)') sec
   sec = char(0)	! byte reservado (= 0)
   write (4, '(a)') sec
   !
   !			PREPARA E GRAVA OS DADOS.
   !
   !
   NBITSTOPUT=0
   oct1= ''
   !
   i=0
1000 continue
   i=i+1
   NBITSTOPUT=NBYTESTOPUT(i)*8+NRESTABITS(i)
   IF(NBITSTOPUT.LT.8)THEN
     oct1=IBITS(values(i),0,8)+oct1
     NBITSTOPUT=8 - NBITSTOPUT
   ENDIF

!
!		    PREPARA E GRAVA A SECAO 5
!

 sec5 = '7777'
 write (4, '(a)') sec5

!
!		     FECHA ARQUIVO E RETORNA
!
 
 close(4)

 RETURN
 
END SUBROUTINE GRAVABUFR2
