!** 
!   MODULE MGRADSPCD
!
!   MODULO PARA GRAVACAO DE DADOS DAS PCDS EM ARQUIVOS PARA LEITURA
!   NO GRADS
!
!   
!   ROTINAS PUBLICAS
!
!     GRAVA_GRADS - grava uma linha de dados binarios do grads
!     GRAVA_GRADSCTL - grava o aquivo descritor (ctl) de um aquivo do grads
!
!   ROTINAS  PRIVATIVAS 
!      grdtdef _ funcao que rotorna a data/hora no formato do ctl do grads
!      
! *   


!   VERSAO ORIGINAL: Sergio H. S. Ferreira 12/12/2003
!
!   MODIFICACOES: 
!
!
!  





MODULE MGRADSPCD

 USE MREADPCD
 USE stringflib

 implicit none


 PRIVATE
 
 real,parameter :: missing=9999.9
 
 PUBLIC GRAVA_GRADS
 PUBLIC GRAVA_GRADSCTL
 
 
 CONTAINS




! ---------------------------------------------------------------------
!
!			   SUBROUTINE  GRAVA_GRADS
!
!			  VERSAO 0.2 - 22/10/2003
!
!	       GRUPO DE PRE-PROCESSAMENTO DE DADOS DO CPTEC
!
! ---------------------------------------------------------------------
!
!  OBJETIVO :
!
!	Esta rotina recebe os valores e dados de uma PCD, e os grava 
!	em formato Binario para Leitura no Grads
!
!
!  VERSAO ORIGINAL: Sergio H. S. Ferreira  22/10/2003	
!
!
!  MODIFICACOES:
!	
!     Sergio - 11/12/2003
!     - Acrescentado a gravacao da pressao reduzida ao nivel do mar
!
! ---------------------------------------------------------------------

!
! PARAMETROS DE ENTRADA IDPCD OBS...

 SUBROUTINE GRAVA_GRADS(UN,IdPCD,Obs,HObs,NObs)


!
!		 DEFINICOES DAS VARIAVEIS DE ENTRADA
!

 type(MREADPCD_Id), intent(in):: IdPCD			! Identificacao de uma 							! PCD (Lat,Lon,etc..)
 
 type(MREADPCD_Date), intent(in):: HObs      		! Data e Hora de uma 
 							! observacao de PCD 
 type(MREADPCD_Obs),dimension(29), intent(in)::Obs 	! Dados Observacionais 

 INTEGER,INTENT(IN)::un,NObs
 					
! Variaveis Locais

 CHARACTER*8 :: STID
 Integer:: NFLAG,NLEV,i,N
 REAL::RLAT,RLON,TIM
 REAL :: DIR,VEL   ! DIRECAO E VELOCIDADE DO VENTO
 REAL :: Ps        ! PRESSAO A SUPERFICIE (hPa)
 REAL :: Ts,tx,tn  ! TEMPERATURA (C) bulbo seco, maxima e minima
 REAL :: UR        !UMIDADE RELATIVA (%)
 REAL :: rglob     ! Radioacao Global 
 REAL :: CX,CY     ! COMPONETES DO VENTO
 REAL :: PLUV      ! PRECIPITACAO 
 REAL :: pnmm      ! PRESSAO REDUZIDA AO NIVEL DO MAR
 
 
      NFLAG = 1
      NLEV = 1
      TIM=0.0
      
      
      CX=missing
      CY=missing
      Ps=missing
      Ts=missing
      Tx=missing
      tn=missing
      pluv=missing
      UR=missing
      rglob=missing
      pnmm=missing
      VEL=NULL
      DIR=NULL  		
       
       Do i=1,Nobs
 	if (Obs(i)%BUFRCOD==011012) then
 	  if(Obs(i)%valor /= NULL) VEL = Obs(i)%valor*2 ! vento (em nos)
 	 end if 
	 
	 if (Obs(i)%BUFRCOD==011011) then
 	   if(Obs(i)%valor /= NULL) DIR = Obs(i)%valor	
 	end if 
	
	if (Obs(i)%BUFRCOD==010004) then
 	   if(Obs(i)%valor /= NULL) Ps = Obs(i)%valor
 	end if 
	
	if (Obs(i)%BUFRCOD==012001) then
 	   if(Obs(i)%valor /= NULL) Ts= Obs(i)%valor	
 	end if 
	
	if (Obs(i)%BUFRCOD==013003) then
 	   if(Obs(i)%valor /= NULL) UR= Obs(i)%valor	
 	end if 
	
	if (Obs(i)%BUFRCOD==013020) then
 	   if(Obs(i)%valor /= NULL) Pluv= Obs(i)%valor	
 	end if
	 
	 if (Obs(i)%BUFRCOD==012016) then
 	   if(Obs(i)%valor /= NULL) Tx= Obs(i)%valor	
 	end if
	
	if (Obs(i)%BUFRCOD==012017) then
 	   if(Obs(i)%valor /= NULL) Tn= Obs(i)%valor	
 	end if
	
	if (Obs(i)%BUFRCOD==014021) then
 	   if(Obs(i)%valor /= NULL) rglob= Obs(i)%valor	
 	end if
	if (Obs(i)%BUFRCOD==010051) then
 	   if(Obs(i)%valor /= NULL) pnmm= Obs(i)%valor	
 	end if
        End do
	
	
	
! Converte direcao e velocidade do vento para componetes meridional e zonal

        if ((DIR/=NULL).AND.(VEL/=NULL)) then 
	    CX = -VEL * SIND(DIR)
            CY = -VEL * COSD(DIR)
      	end if
	
	 		
        STID=str$(IdPCD%PCDCOD)
				
	WRITE (un) STID,IdPCD%Lat,IdPCD%LON,TIM,NLEV,NFLAG
	WRITE (un) PNMM,PS,Ts,UR,CX,CY,Tx,Tn,Pluv,rglob
	


 END subroutine  GRAVA_GRADS
 
 
 !**
 ! SUBROUTINE GRAVA_GRADSCTL
 !
 ! Subroutina para gerar o arquivo descritor (ctl) para o GRADS
 !  
 ! NOTA: Esta subroutina utiliza a funcao grdtdef para gerar a
 !       linha de data e hora em formato apropriado 
 !  
 !  VERSAO ORIGINAL: Sergio H. S. Ferreira 12/12/2003
 !
 !  MODIFICACOES:
 !
  
 
 SUBROUTINE GRAVA_GRADSCTL(filename,hour,day,month,year)

 ! Variaveis de entrada
 
      character(len=*),intent(in)::filename
      integer:: hour,day,month,year
 
 ! Variaveis locais
 
 	Character(len=255)::ctlname
	integer ::i
	
 ! Gravar arquivo descritor (CTL)
 
   ctlname=trim(filename)//'.ctl'
   
   open(55,file=ctlname,status='unknown')
       
	write(55,1001)trim(filename)    
   1001 format('DSET ^',a,'.bin')
          
	write(55,1002)    
   1002 format('DTYPE station')
   
        write(55,1003)trim(filename)    
   1003 format('STNMAP ^',a,'.map')
    
        write(55,1004) missing
   1004 format('UNDEF ',f6.1)
   
        write(55,1005)
   1005 format('TITLE INPE-PCD')
   
        write(55,1006) grdtdef(hour,day,month,year)
   1006 format('TDEF 1 linear ',a,' 3hr')
   
        write(55,1007)
   1007 format('VARS 10')
   
        write(55,1008)
   1008 format('pnm 0 99 pressao reduzida ao nivel medio do mar (hPa)')
   
        write(55,1009)
   1009 format('ps  0 99 pressao `a superfice (hPa)')
   
        write(55,1010)
   1010 format('ts  0 99 temperatura bulbo seco (c)')
        
	write(55,1011)
   1011 format('ur  0 99 Umidade relativa do ar (%)')
        
	write(55,1012)
   1012 format('u   0 99 Componente meridional  do vento (nos)') 
        
	write(55,1013)
   1013 format('v   0 99 Componente zonal do  vento (nos)')
        
	write(55,1014)
   1014 format('tx  0 99 temperatura maxima (c)')
        
	write(55,1015)
   1015 format('tn  0 99 temperatura minima (c)')
        
	write(55,1016)
   1016 format('pl  0 99 precipitacao (mm)')
        
	write(55,1017)
   1017 format('rad 0 99 radiacao global') 
        
	write(55,1018)
   1018 format('ENDVARS')
   
   
   close(55)
 
 end subroutine GRAVA_GRADSCTL
 
 !**
 ! Function grdtdef
 !  
 !  Funcao que retorna a data e hora no formato do ctl do grads
 !  (character com 19 posicoes)
 !
 !  Versao original: Sergio H S Ferreira 11/12/2003
 !
 !**
 
 
  function grdtdef(hour,day,month,year);character(len=19)::grdtdef
  
  integer,intent(in)::hour,day,month,year
  character(len=3),dimension(12)::nm
  character(len=19)::a
  nm(1)="Jan"
  nm(2)="Feb"
  nm(3)="Mar"
  nm(4)="Apr"
  NM(5)="May"
  nm(6)="Jun"
  nm(7)="Jul"
  nm(8)="Ago"
  nm(9)="Set"
  nm(10)="Oct"
  nm(11)="Nov"
  nm(12)="Dec"

  write(a,10)hour,day,nm(month),year
10 format(i2.2,":00Z",i2.2,a3,i4)
  grdtdef=a
 end function
 
 
END MODULE
