!**
! MREADPCD  
!
! MODULO DE LEITURA DOS ARQUIVOS TEXTO DAS PCDS DO DSA 
! 
! Os arquivos de PCDs sao arquivos texto em que cada linha corresponde a uma 
! observacao colida pela PCD em um determinada data e horarios.
! 
!  As rotinas contidas neste modulo sao 
!	 
!    Init_MREADPCD    Inicializa este modulo e ler os aquivos de estacao das PCDS
!    Id_MREADPCD      Identifica a PCD (tipo e posicao geografica)
!    ReadOBS_MREADPCD Ler os dados de PCDs
!
!
!    Versao Original: Sergio H. S. Ferreira Outubro de 2003 
!  
!
! MODIFICACOES:     
! 
! Modificado por Waldenio - 30/09/2003 a 01/10/2003
!
! Mudei a definicao de algumas variaveis !!!
! Mudei algumas conversoes de unidades que devem ser feitas apenas na codificacao BUFR
! Mudei descritores BUFR
!
!
! Modificado por Sergio  11/12/2003
! 
! - Acrescentado calculo da pressao reduzida ao nivel do mar
! - Acrescentado calculo da precipitacao acumulada em 3 horas
! - Modificado  para fornecer os dados do ultimo horario de cada arquivo
! - Implementado uma lista circirlar para manter em memorio os ultimos 5 horarios de dados 


MODULE MREADPCD
	


 USE STRINGFLIB
 USE DATELIB
 
 implicit none
 
 PUBLIC





!* Declacao das Estrutura dos dados 
 
type MREADPCD_Id
 Integer:: PCDCOD	             	! CODIGO DA PCD
 character(len=20)::NAME         	! Nome da Localidade ou Estacao
 Real:: Lat,lon				! Coordenadas Horizontais da PCD
 real::	Ha				! Altitude da Estacao ou localidade
end	type MREADPCD_Id


type MREADPCD_Date
   Integer:: MES,DIA,ANO,HORA,MIN  ! DATA E HORA 
end type MREADPCD_date  


type  MREADPCD_Obs
 real:: Valor				! Valor Observado
 integer:: BUFRCOD			! Codigo BUFR ou Nao  que identifica o tipo variavel
 integer:: QC 
end type  MREADPCD_Obs







!* Variaveis Publicas 

 integer(1),parameter::PCD_METEOROLOGICA=1
 integer(1),parameter::PCD_HIDROLOGICA=2
 integer(1),parameter::PCD_BOIA=3
 





!* Variaveis Privadas

    integer,dimension(3,29),private::BUFRCOD
    type(MREADPCD_ID),dimension(500),private:: stations
    integer,private:: nstations ! numero todal de estacoes cadastradas 


  
 CONTAINS

 !** Surotina Init_MREADPCD
 !
 !   Esta subrotina Inicializa as variaveis e tabelas deste modulo,
 !   incluindo a tabela de estacoes e a tabela de codigos BUFR
 !**  
 
 subroutine Init_MREADPCD

 	 
 !* 1.-  Declara variaveis Locais

    character(len=255)::linha,nome$
	integer :: cod
	real:: lat,lon,alt


 !* 2. - INICIALIZA TABELA BUFRCOD
 !
 !     BUFRCOD(TIPOPCD,Coluna) 
 !         
 !     Tabela que associa a cada  cada uma das colunas dos arquivos PCD.TXT
 !     e a cada um dos tipos de PCD um codigo da variavel (PSEUDO-CODIGO BUFR)
 !           
 !     Onde: 
 !         TIPOPCD = tipo de PCD (1,2 ou 3) 
 !         Coluna = Posicao da coluna em que aparece a variavel no arquivo de PCD
 !
 !	      
 !      Obs.: 
 !         Caso BUFRCOD(TIPOPCD,Coluna)=0 --> Sg. Nenhuma variavel atribuida a esta posicao
 !
 
    BUFRCOD(:,:)=0

 !* PCD METEOROLOGICA 

	BUFRCOD( PCD_METEOROLOGICA, 1)=1      !N/A
	BUFRCOD( PCD_METEOROLOGICA, 2)=2      !Bateria
	BUFRCOD( PCD_METEOROLOGICA, 3)=3      !CorrPSol
	BUFRCOD( PCD_METEOROLOGICA, 4)=101    !DirVelVentoMax
	BUFRCOD( PCD_METEOROLOGICA, 5)=011011 !DirVento
	BUFRCOD( PCD_METEOROLOGICA, 6)=150    !Nivel do Pluviometro 
	!Nota o BUFRCOD=150 e modificado para 13020 apos calculo da precipitacao acumulada em readOBS
	BUFRCOD( PCD_METEOROLOGICA, 7)=201    !Pluvio24H-12GMT
	BUFRCOD( PCD_METEOROLOGICA, 8)=010004 !PressaoAtm - nivel estacao
	BUFRCOD( PCD_METEOROLOGICA, 9)=401    !RadSolAcum
	BUFRCOD( PCD_METEOROLOGICA,10)=014021 !RadSolGlob
	BUFRCOD( PCD_METEOROLOGICA,11)=403    !RadSolInc
	BUFRCOD( PCD_METEOROLOGICA,12)=404    !RadSolPAR
	BUFRCOD( PCD_METEOROLOGICA,13)=405    !RadSolRef
	BUFRCOD( PCD_METEOROLOGICA,14)=012001 !TempAr
	BUFRCOD( PCD_METEOROLOGICA,15)=012016 !TempMax
	BUFRCOD( PCD_METEOROLOGICA,16)=012017 !TempMin,
	BUFRCOD( PCD_METEOROLOGICA,17)=601    !UmidInt
	BUFRCOD( PCD_METEOROLOGICA,18)=013003 !UmidRel
	BUFRCOD( PCD_METEOROLOGICA,19)=011012 !VelVento10m
	BUFRCOD( PCD_METEOROLOGICA,20)=112    !VelVento2m
	BUFRCOD( PCD_METEOROLOGICA,21)=113    !VelVento3m
	BUFRCOD( PCD_METEOROLOGICA,22)=114    !VelVento5m
	BUFRCOD( PCD_METEOROLOGICA,23)=115    !VelVentoMax
	BUFRCOD( PCD_METEOROLOGICA,24)=701    !ContAguaSolo100
	BUFRCOD( PCD_METEOROLOGICA,25)=702    !ContAguaSolo200
	BUFRCOD( PCD_METEOROLOGICA,26)=703    !ContAguaSolo400
	BUFRCOD( PCD_METEOROLOGICA,27)=801    !TempSolo100
	BUFRCOD( PCD_METEOROLOGICA,28)=802    !TempSolo200
	BUFRCOD( PCD_METEOROLOGICA,29)=803    !TempSolo400
	!	    
	!* PCD HIDROLOGICA
	!	    
	BUFRCOD( PCD_HIDROLOGICA, 1)=1       !N/A,
	BUFRCOD( PCD_HIDROLOGICA, 2)=2       !Bateria
	BUFRCOD( PCD_HIDROLOGICA, 3)=30      !CodifEixo
	BUFRCOD( PCD_HIDROLOGICA, 4)=3       !CorrPsol
	BUFRCOD( PCD_HIDROLOGICA, 5)=31      !gcCsol,
	BUFRCOD( PCD_HIDROLOGICA, 6)=32      !LgcTsol
	BUFRCOD( PCD_HIDROLOGICA, 7)=33      !NivRegua
	BUFRCOD( PCD_HIDROLOGICA, 8)=150     !Nivel do Pluviometro
	!Nota o BUFRCOD=150 e modificado para 13020 apos calculo da precipitacao acumulada em readOBS

	BUFRCOD( PCD_HIDROLOGICA, 9)=202     !Pluvio24H-12GMT
        ! BUFRCOD( PCD_HIDROLOGICA,10)=010004  !SPressao - estacao
	BUFRCOD( PCD_HIDROLOGICA,10)=000000  !SPressao - estacao ! LAC
	BUFRCOD( PCD_HIDROLOGICA,11)=4       !TempInt
	BUFRCOD( PCD_HIDROLOGICA,12)=5       !UmidInt
	
	!*PCD BOIAS	
	
	BUFRCOD( PCD_BOIA,1)=1   !N/A
	BUFRCOD( PCD_BOIA,2)=0   !CHK
	BUFRCOD( PCD_BOIA,3)=2   !Battery
	BUFRCOD( PCD_BOIA,4)=504 !Sea Surface Temp
	BUFRCOD( PCD_BOIA,5)=0   !Submergence
	BUFRCOD( PCD_BOIA,6)=302 !Barometric Press
	
 
!* 4. -  Inicializa Tabela de Estacoes 
!
 
	nstations=0

    OPEN(1,file='cadpcds.dat',status='OLD')
	read(1,*)linha

10  	read(1,*,end=999)cod,lat,lon,alt,Nome$
	    nstations=nstations+1
		stations(nstations).pcdcod=cod
		stations(nstations).lat=lat
		stations(nstations).lon=lon
		stations(nstations).Ha=alt
		stations(nstations).Name=Nome$
	  

	goto 10
999  CONTINUE
	 Print *, "Numero de estacoes cadastradas = ",nstations

END subroutine Init_MREADPCD

  
  
    

!**  Subroutina Idtype_MREADPCD
!
!  Esta rotina ler o cabecalho da PCD e identifica o tipo da PCD
!  que esta sendo lida. 
!  
!  Caso nao seja possivel identifica o tipo da PCD  entao TIPOPCD = 0 
! 
!
!  As PCDS podem ser Meteorologicas, Hidrologicas ou Boias
!  Considera-se que: 
!    a)  somente as PCDS Hidrologicas terao dados de Nivel de Regua
!    b)  somente as PCDS Boias terao valor de Submergence
!
!  As demais PCDS sao consideradas como PCDS Meteorologicas 
!
!
!   Parametros de Entrada
!    Un = Unidade de Leitura
!
!   Parametros de Saida
!    TIPOPCD = Codigo que identifica o tipo da PCD
!**

subroutine Idtype_MREADPCD(Un,TIPOPCD)
   
Integer,intent(in):: Un
Integer(1),intent(out)::TIPOPCD

!* Variaveis locais

character(255)::linha

!* Inicializacao da variaveis

   TIPOPCD=0

!*

   read(un,'(a)',end=100)linha        
  

     TIPOPCD=PCD_METEOROLOGICA

     if (index(linha,"NivRegua")>0)    TIPOPCD=PCD_HIDROLOGICA
     if (index(linha,"Submergence")>0) TIPOPCD=PCD_BOIA

100 continue

end subroutine Idtype_MREADPCD





!** Subroutine Idstation_MREADPCD
!
!   Esta subrotina  localiza uma estacao PCDS no arquivo de estacao
!   e obtem as informacoes de latitude,longitude,altitude e nome da estacao 
!  
!   Caso a Estacao Nao seja encontrada retorna o valor Null en IdPCD
!**

subroutine IdStation_MREADPCD(NumPCD,IdPCD)
  Integer,intent(in)::NumPCD ! Numero da PCD fornecido
  type(MREADPCD_Id),intent(out)::IdPCD	! Dados Cadastrais relativos ao NumPCD

  Integer::i
  
  IdPCD%PCDCOD=Null
  IdPCD%Name=""
  IdPCD%Lat=Null
  IdPCD%Lon=Null
  IdPCD%Ha=Null
  
 
  do i=1,nstations

   if (NumPCD==stations(i).PCDCOD) then 
      
	  IdPCD%PCDCOD = stations(i)%PCDCOD
	  IdPCD%NAME   = stations(i)%NAME
	  IdPCD%Lat    = stations(i)%Lat
	  IdPCD%Lon    = stations(i)%Lon
	  IdPCD%Ha     = stations(i)%Ha
     
	  exit 

  end if
  
  end do

 

END subroutine IdStation_MREADPCD


 !** ReadOBS_MREADPCD
 !  
 !    Esta rotina ler a linha corrente de um arquivo de  observacao de PCD
 !    e retorna os valores observacionais lisdos na  matriz OBS
 !
 !	   Parametros de Entrada
 !       
 !       UN = unidade de leitura
 !       TIPOPCD = Identificacao do Tipo da PCD
 !
 !    Parametros de saida
 !       OBS = Matriz de Observacoes lidas (leitura corrente)
 !       NOBS = Numero de dados dentro de OBS. Se a  data da observacao
 !              nao for encontrada entao NOBS=0
 !
 !
 !
  
 !** 


 

 subroutine ReadOBS_MREADPCD(UN,TIPOPCD,IdPCD,HObs,Obs,NObs)


 !* Parametros de entrada

 Integer,intent(in):: UN			       ! Unidade de Leitura
 Integer(1),intent(in):: TIPOPCD		       ! Tipo da PCD
 type(MREADPCD_id),intent(in)::IdPCD                   ! Identificacao da PCD
 type(MREADPCD_Date),intent(inout):: HObs 	       ! Data e Hora da Observacao
 
 !* Parametros de saida
 
 
 type(MREADPCD_Obs),dimension(:),intent(out):: obs  	! valores observacionais
 integer,intent(out):: nobs				! Numero de variaveis das observacoes


 !* Variaveis locais
 type(MREADPCD_date)::HOBS_aux             ! Variavel auxiliar para data e hora da obs
 character(len=255):: linha                ! uma linha de dados	do arquivo
 character(len=255),dimension(50) :: cols  ! vetor contendo as colunas de dados de uma linha do arquivo
 integer :: ncols                          ! nunmero de colunas de cols
 real    :: BATERIA                        ! Tensao da bateria em volts
 integer :: i
 real    :: valor
 real    :: TT,TT_12                       ! Temperatura do ar (ATUAL E 12 HORAS ANTES)
 real    :: UR,UR_12                       ! Umidade relativa  (ATUAL E 12 HORAS ANTES)
 real    :: Tv                             ! Temperatura Virtual Media
 real    :: Ps,Ps_12                       ! Pressao a superficie (Atual e 12 horas antes)
 real    :: PNMM  			   !Pressao reduzida ao Nivel Medio do Mar					    
 !integer :: H,H_12,Hant                    ! Hora (ATUAL E 12 HORAS ANTES e Horario Anterior)
 real    :: NPluv,NPluv_Hant               ! Nivel do Pluviometro (Atual e Anterior)
 real    :: Prec                           ! Precipitacao acumulada 
 logical :: HLOC                           ! HLOC=.true. se Localizou os observacoes do horario solicitado em HOBS
 real*8  :: DJULIAN                        ! Data Juliana
 real*8  :: DJULIAN_ant                    ! Data Juliana da observacao anterior
!* Define uma lista Circular de 4 elementos para calculo da temperatura
!  virtual media das ultimas 12 horas

 type elemento_lista
   real :: DJULIAN   ! DATA JULIANA EM DIAS E FRACAO DE DIAS 
   real :: TT        ! Temperatura do Ar
   real :: UR        ! Umidade relativa do ar
   real :: Ps        ! Pressao de superficie
   real :: Npluv     ! Nivel do pluviometro 
 end type
 
 type(elemento_lista),dimension(1:5)::lista_circular
 integer:: Ponteiro_lista
 
 
 !*  Inicializar variaveis e estruturas de dados
 
 !* a) zerar Lista_circular
 
 do Ponteiro_lista=1,5
	  	  
	   Lista_circular(ponteiro_lista)%DJULIAN=0
	   Lista_circular(ponteiro_lista)%TT=null
	   Lista_circular(ponteiro_lista)%UR=null
	   Lista_circular(ponteiro_lista)%Ps=null
	   Lista_circular(ponteiro_lista)%NPluv=null
 end do
 Ponteiro_lista=0
 
 ! b) Zerar estrutura da variavel obs
          obs(:)%Valor=null				
          obs(:)%BUFRCOD=null			
          NOBS=0
 
 ! c) INICIAR outras variaveis 
      HLOC=.false.
 

!* Processa a leitura do arquivo e armazenamento de valores na lista_circular 
 


10 read(UN,'(a)',end=100)linha

   call sep_substrings(linha,",",cols,ncols)  ! separa as colunas de cada linha conforme as virgulas 
  
    
 
   ! a) Obtem data, hora e tensao da bateria
 

        HObs_aux%MES=val(cols(1)(1:2)) ! LAC
	HObs_aux%DIA=val(cols(1)(4:5)) ! LAC
	HObs_aux%ANO=val(cols(1)(7:10))
	HObs_aux%HORA=val(cols(1)(12:13))
	Hobs_aux%MIN =val(cols(1)(15:16))
	
	BATERIA=VAL(cols(2))
	
    ! b) Armazena os ultimos 4 valores de Temperatura e Umidade 
    ! na lista circular 
  
      tt=null
      ur=null
      ps=null
      Npluv=null
      do i=3,ncols
       if (BUFRCOD(TIPOPCD,i)==12001) TT=val(cols(i))
       if (BUFRCOD(TIPOPCD,i)==13003) UR=val(cols(i))
       if (BUFRCOD(TIPOPCD,i)==10004) Ps=val(cols(i))
       if (BUFRCOD(TIPOPCD,i)==150) NPluv=val(cols(i))  
      end do
      
      Ponteiro_lista=Ponteiro_lista+1
      if (ponteiro_lista>5) Ponteiro_lista=1
	 
      Lista_circular(ponteiro_lista)%DJULIAN=fjulian(HObs_aux%ANO,HObs_aux%MES,HObs_aux%DIA,HObs_aux%HORA,0,0)
      Lista_circular(Ponteiro_lista)%TT = TT
      Lista_circular(Ponteiro_lista)%UR = UR
      Lista_circular(Ponteiro_lista)%Ps = Ps
      Lista_circular(Ponteiro_lista)%NPluv = NPluv
    
    if ((HObs_aux%DIA==HObs%DIA).and.(HObs_aux%MES==HObs%MES).and.(HObs_aux%ANO==HObs%ANO).and.(HObs_aux%HORA==HObs%HORA)) THEN 
       HLOC=.TRUE.
       GOTO 100
    END IF
     
    GO TO 10
    
    
100 CONTINUE   
    
  
    NOBS=0
    IF (HLOC==.false.) goto 1000
      
    DJULIAN=fjulian(HObs%ANO,HObs%MES,HObs%DIA,HObs%HORA,0,0)
   
   !* Calculo da Pressao Reduzida ao Nivel do Mar (PNMM)

    
       PNMM=null
       TT_12=null
       UR_12=null
       Ps_12=null
       
      
    
      if (ps/=null) then 
      
        ! a) Procura por dados de 12 horas antes na lista_circular
	
	DJULIAN_ant=DJULIAN-(12.0/24.0)
	
	!H_12=HObs%HORA-12
	!if (H_12<0) H_12=24+H_12
	
	do Ponteiro_lista=1,5
	  if (Lista_circular(ponteiro_lista)%DJULIAN==DJULIAN_ant) then
	   TT_12=Lista_circular(ponteiro_lista)%TT
	   UR_12=Lista_circular(ponteiro_lista)%UR
	   Ps_12=Lista_circular(ponteiro_lista)%Ps
	   exit 
	  end if
	end do
	
	! b) Calculo da PNMM
	
	if ((tvirt(TT,UR,Ps)/=null).and.(tvirt(TT_12,UR_12,Ps_12)/=null)) then 
	
	   Tv = (Tvirt(TT,UR,Ps)+Tvirt(TT_12,UR_12,Ps_12))/2 
           
           Tv = Tv - (0.0065/2)*(0.0 - idPCD%Ha)

	   PNMM=Ps*exp(idPCD%Ha/(29.3*(Tv+273.16)))   
	
	elseif ((TT/=null).AND.(Ps/=null)) then

           Tv = TT - (0.0065/2)*(0.0 - idPCD%Ha)
           
           PNMM=Ps*exp(idPCD%Ha/(29.3*(Tv+273.16)))

	end if
	
        
     endif
    
    
    !* Calculo da precipitacao acumulada nas ultimas 3 horas 
    
    
    
    
    NPluv_HAnt=null
    Prec=null
    
    If (Npluv /= null) then 
    
      !* a) Busca na fila circular
    
    
      DJULIAN_Ant=DJULIAN-(3.0/24.0)
      !if (HAnt<0) HAnt=24+HAnt
    
      if (NPluv/=null) then 
       do Ponteiro_lista=1,5
   !SH       print *,Ponteiro_lista,Lista_circular(ponteiro_lista)%hora,HObs_aux%DIA
	  if (Lista_circular(ponteiro_lista)%DJULIAN==DJULIAN_Ant) then
	   NPluv_HAnt=Lista_circular(ponteiro_lista)%NPluv
	   exit 
	  end if
	end do
    end if
    
    
   !  b) Calculo da precipitacao
    if (NPluv_Hant/=null) then 
    
   
   !  b-1)  Caso normal
      Prec=Npluv-Npluv_Hant  
      
   !  b-2) Caso ocorra virada do  pluviometro
      If (prec<0) then 
      
        If ((NPluv_Hant>445.0).and.(Npluv_HAnt<=655.0)) then 
	 
	  Prec=655.0-Npluv_Hant+NPluv        ! Para Pluviometo com Limite em 655
	
	elseif ((NPluv_Hant>1847.0).and.(Npluv_HAnt<=2047.0)) then 
	
	  Prec=2047.0-Npluv_HAnt+NPluv       ! Para Pluviometo com Limite em 2047
        
	elseif ((NPluv_Hant>3895.0).and.(Npluv_HAnt<=4095)) then 
	
	  Prec=4095.0-Npluv_HAnt+NPluv       !Para Pluviometro com Limite em 4095
	
	else  
	  
	  ! Para caso de Pluviometro zerado foram dos limites estabelecidos
	  ! e na falta de se conhecer este limite 
	  ! utiliza-se valor atual como uma medida aproximada 
	  ! da precipitacao
	  
	  
           Prec=Npluv                         
	   Write(2,20)IdPCD%PCDCOD
 20        format('Aviso! Pluviometro zerado fora dos limites conhecidos na estacao',I6)
	end if
	
      end if	
	
      if (prec>200) then    
	   write(2,21)IdPCD%PCDCOD
	   write(2,22)Prec
 21        format('Aviso! Precipitacao acima de 200mm/3h em',I6)	    
 22	   format('       Prec =',f10.1,'mm/3h  **Valor Eliminado**') 
	   Prec=null
	 
      end if 
      end if
    
    end if
    
   !* 
   !    Passagem de valores para o vetor de observacoes 
   
   ! a) Identificacao das observacoes
          
      	  OBS(1)%valor=val(cols(1)(1:2)) ! mes 
      	  OBS(1)%BUFRCOD=4002
          OBS(1)%QC=70
          !
	  OBS(2)%valor=val(cols(1)(4:5)) ! dia 
      	  OBS(2)%BUFRCOD=4003
	  OBS(2)%QC=70
          !
          OBS(3)%valor=val(cols(1)(7:10)) ! ano
      	  OBS(3)%BUFRCOD=4001
          OBS(3)%QC=70
          !
	  OBS(4)%valor=val(cols(1)(12:13)) ! hora
      	  OBS(4)%BUFRCOD=4004
          OBS(4)%QC=70
          !
          OBS(5)%valor=val(cols(1)(15:16)) ! min
      	  OBS(5)%BUFRCOD=4005
          OBS(5)%QC=70
          !
	  OBS(6)%valor=IdPCD%Lat
	  OBS(6)%BUFRCOD=5001
	  OBS(6)%QC=70
          !
          OBS(7)%valor=IdPCD%Lon
	  OBS(7)%BUFRCOD=6001
	  OBS(7)%QC=70
          !
          OBS(8)%valor=IdPCD%Ha
	  OBS(8)%BUFRCOD=7001
	  OBS(8)%QC=70
          !
          OBS(9)%valor=IdPCD%PCDCOD
	  OBS(9)%BUFRCOD=1015
	  OBS(9)%QC=70
          !

  ! b)  Medidas diretas 



 
  
  NOBS=9

  DO I=3,NCOLS
    
	valor=val(cols(i))

    	if ((valor/= NULL).and.(BUFRCOD(TIPOPCD,I)>1000)) then 
	
	  NOBS=NOBS+1
      	  OBS(NOBS)%valor=valor 
      	  OBS(NOBS)%BUFRCOD=BUFRCOD(TIPOPCD,I)
	  OBS(NOBS)%QC=70

	
	end if
	


  END DO
  
  
  ! c) Medidas derivadas 
  
  ! c-1)Pressao reduzida ao nivel do mar
   
  if (PNMM/= NULL) then 
      NOBS=NOBS+1
      OBS(NOBS)%valor=PNMM
      OBS(NOBS)%BUFRCOD=10051
      IF ((tvirt(TT,UR,Ps)/=null).and.(tvirt(TT_12,UR_12,Ps_12)/=null)) then
      
        OBS(NOBS)%QC=70
	
      elseif ((TT/=null).AND.(Ps/=null)) then
      
        OBS(NOBS)%QC=50
     
      END IF  
  END IF
  
! c-2) Precipitacao Acumulada

  If (Prec/=Null) then 
      NOBS=NOBS+1
      OBS(NOBS)%valor=Prec
      OBS(NOBS)%BUFRCOD=13020
      OBS(NOBS)%QC=70
  end if
       
  1000 continue
       
END SUBROUTINE	READOBS_MREADPCD


! **
! Funcao TVIRT - 
!  Esta funcao retorna a  temperatura virtual em graus Ceusius em 
!  Funcao de: 
!   TT =   Temperatura do Ar  (C) 
!   Ur =   Umidade relativa (%)
!    P =   Pressao atmosferica no nivel da observacao (hPa)
!
!  Caso UR = Null entao a funcao retonrna a propria temperatura do ar
!            (Admite-se que Tv ~ T)
!
!  Caso TT = Null entao a funcao retorna o valor null
!
!  Caso P = Null considera-se p = 1013.2
!
!  SERGIO H. S. FERREIRA
!
!
	FUNCTION TVirt(TT,UR,P);real::TVirt

  	Real,intent(in)::TT      ! Temperatura do ar (C)
  	Real,intent(in)::UR      ! Umidade relativa (%)
  	Real,Intent(in)::P       ! Pressao Atmosferica (hPa)
 ! Variaveis locais
 
  	real          :: Es      ! tensao maxima do vapor
  	real          :: Tv      ! temperatura virtual 
 	real	      :: q       ! Umidade especifica
 ! Processamento da Funcao 
         
	Tv=null
	
	
  	if (tt/=null) then 
       
          if (ur/=null) then 
       
           	! Calculo da pressao  maxima do vapor sobre a agua
	   	!(formula de Tetens)    
	 
	   	Es=6.1078*10**(7.5*tt/(237.3+tt)) 
               
                ! Calculo da Umidade especifica
		 if ((p>0).and.(p/=null)) then 
		   q = 0.622*(UR*Es/100.0)/P
		 else 
		   q = 0.622*(UR*Es/100.0)/1013.2
		 end if
	      
           	!Calculo da Temperatura Virtual (C)
	 
	   	TV=(TT+273.16)*(1+0.61*q)-273.16
	 
           else 
       
          	! No caso de UR nao ser informado admite-se que TV ~ TT
	  	TV = TT
       
       end if
       end if

 Tvirt= Tv

end FUNCTION TVirt

!======================================================
END MODULE
 


